"""
A few checks to make sure the environment is sane
"""

import errno
import logging
import os
import re
import socket
import stat
import sys

import salt.defaults.exitcodes
import salt.utils.files
import salt.utils.path
import salt.utils.platform
import salt.utils.user
from salt.exceptions import CommandExecutionError, SaltClientError, SaltSystemExit
from salt.log import is_console_configured
from salt.log.setup import LOG_LEVELS

# Original Author: Jeff Schroeder <jeffschroeder@computer.org>


try:
    import win32file
    import salt.utils.win_reg
except ImportError:
    import resource

log = logging.getLogger(__name__)

ROOT_DIR = "c:\\salt" if salt.utils.platform.is_windows() else "/"
DEFAULT_SCHEMES = ["tcp://", "udp://", "file://"]


def zmq_version():
    """
    ZeroMQ python bindings >= 2.1.9 are required
    """
    try:
        import zmq
    except Exception:  # pylint: disable=broad-except
        # Return True for local mode
        return True
    ver = zmq.__version__
    # The last matched group can be None if the version
    # is something like 3.1 and that will work properly
    match = re.match(r"^(\d+)\.(\d+)(?:\.(\d+))?", ver)

    # Fallthrough and hope for the best
    if not match:
        msg = "Using untested zmq python bindings version: '{}'".format(ver)
        if is_console_configured():
            log.warning(msg)
        else:
            sys.stderr.write("WARNING {}\n".format(msg))
        return True

    major, minor, point = match.groups()

    if major.isdigit():
        major = int(major)
    if minor.isdigit():
        minor = int(minor)

    # point very well could be None
    if point and point.isdigit():
        point = int(point)

    if major == 2 and minor == 1:
        # zmq 2.1dev could be built against a newer libzmq
        if "dev" in ver and not point:
            msg = "Using dev zmq module, please report unexpected results"
            if is_console_configured():
                log.warning(msg)
            else:
                sys.stderr.write("WARNING: {}\n".format(msg))
            return True
        elif point and point >= 9:
            return True
    elif major > 2 or (major == 2 and minor > 1):
        return True

    # If all else fails, gracefully croak and warn the user
    log.critical("ZeroMQ python bindings >= 2.1.9 are required")
    if "salt-master" in sys.argv[0]:
        msg = (
            "The Salt Master is unstable using a ZeroMQ version "
            "lower than 2.1.11 and requires this fix: http://lists.zeromq."
            "org/pipermail/zeromq-dev/2011-June/012094.html"
        )
        if is_console_configured():
            log.critical(msg)
        else:
            sys.stderr.write("CRITICAL {}\n".format(msg))
    return False


def lookup_family(hostname):
    """
    Lookup a hostname and determine its address family. The first address returned
    will be AF_INET6 if the system is IPv6-enabled, and AF_INET otherwise.
    """
    # If lookups fail, fall back to AF_INET sockets (and v4 addresses).
    fallback = socket.AF_INET
    try:
        hostnames = socket.getaddrinfo(
            hostname or None, None, socket.AF_UNSPEC, socket.SOCK_STREAM
        )
        if not hostnames:
            return fallback
        h = hostnames[0]
        return h[0]
    except socket.gaierror:
        return fallback


def verify_socket(interface, pub_port, ret_port):
    """
    Attempt to bind to the sockets to verify that they are available
    """

    addr_family = lookup_family(interface)
    for port in pub_port, ret_port:
        sock = socket.socket(addr_family, socket.SOCK_STREAM)
        try:
            sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            sock.bind((interface, int(port)))
        except Exception as exc:  # pylint: disable=broad-except
            msg = "Unable to bind socket {}:{}".format(interface, port)
            if exc.args:
                msg = "{}, error: {}".format(msg, str(exc))
            else:
                msg = "{}, this might not be a problem.".format(msg)
            msg += "; Is there another salt-master running?"
            if is_console_configured():
                log.warning(msg)
            else:
                sys.stderr.write("WARNING: {}\n".format(msg))
            return False
        finally:
            sock.close()

    return True


def verify_logs_filter(files):
    to_verify = []
    for filename in files:
        verify_file = True
        for scheme in DEFAULT_SCHEMES:
            if filename.startswith(scheme):
                verify_file = False
                break
        if verify_file:
            to_verify.append(filename)
    return to_verify


def verify_log_files(files, user):
    """
    Verify the log files exist and are owned by the named user.  Filenames that
    begin with tcp:// and udp:// will be filtered out. Filenames that begin
    with file:// are handled correctly
    """
    return verify_files(verify_logs_filter(files), user)


def _get_pwnam(user):
    """
    Get the user from passwords database
    """
    if salt.utils.platform.is_windows():
        return True
    import pwd  # after confirming not running Windows

    try:
        return pwd.getpwnam(user)
    except KeyError:
        msg = (
            "Failed to prepare the Salt environment for user {}. The user is not"
            " available.".format(user)
        )
        if is_console_configured():
            log.critical(msg)
        else:
            print(msg, file=sys.stderr, flush=True)
        sys.exit(salt.defaults.exitcodes.EX_NOUSER)


def verify_files(files, user):
    """
    Verify that the named files exist and are owned by the named user
    """
    if salt.utils.platform.is_windows():
        return True

    # after confirming not running Windows
    pwnam = _get_pwnam(user)
    uid = pwnam[2]

    for fn_ in files:
        dirname = os.path.dirname(fn_)
        try:
            if dirname:
                try:
                    os.makedirs(dirname)
                except OSError as err:
                    if err.errno != errno.EEXIST:
                        raise
            if not os.path.isfile(fn_):
                with salt.utils.files.fopen(fn_, "w"):
                    pass

        except OSError as err:
            if os.path.isfile(dirname):
                msg = "Failed to create path {}, is {} a file?".format(fn_, dirname)
                raise SaltSystemExit(msg=msg)
            if err.errno != errno.EACCES:
                raise
            msg = 'No permissions to access "{}", are you running as the correct user?'.format(
                fn_
            )
            raise SaltSystemExit(msg=msg)

        except OSError as err:  # pylint: disable=duplicate-except
            msg = 'Failed to create path "{}" - {}'.format(fn_, err)
            raise SaltSystemExit(msg=msg)

        stats = os.stat(fn_)
        if uid != stats.st_uid:
            try:
                os.chown(fn_, uid, -1)
            except OSError:
                pass
    return True


def verify_env(
    dirs, user, permissive=False, pki_dir="", skip_extra=False, root_dir=ROOT_DIR
):
    """
    Verify that the named directories are in place and that the environment
    can shake the salt
    """
    if salt.utils.platform.is_windows():
        return win_verify_env(
            root_dir, dirs, permissive=permissive, skip_extra=skip_extra
        )

    # after confirming not running Windows
    pwnam = _get_pwnam(user)
    uid = pwnam[2]
    gid = pwnam[3]
    groups = salt.utils.user.get_gid_list(user, include_default=False)

    for dir_ in dirs:
        if not dir_:
            continue
        if not os.path.isdir(dir_):
            try:
                with salt.utils.files.set_umask(0o022):
                    os.makedirs(dir_)
                # If starting the process as root, chown the new dirs
                if os.getuid() == 0:
                    os.chown(dir_, uid, gid)
            except OSError as err:
                msg = 'Failed to create directory path "{0}" - {1}\n'
                sys.stderr.write(msg.format(dir_, err))
                sys.exit(err.errno)

        mode = os.stat(dir_)
        # If starting the process as root, chown the new dirs
        if os.getuid() == 0:
            fmode = os.stat(dir_)
            if fmode.st_uid != uid or fmode.st_gid != gid:
                if permissive and fmode.st_gid in groups:
                    # Allow the directory to be owned by any group root
                    # belongs to if we say it's ok to be permissive
                    pass
                else:
                    # chown the file for the new user
                    os.chown(dir_, uid, gid)
            for subdir in [a for a in os.listdir(dir_) if "jobs" not in a]:
                fsubdir = os.path.join(dir_, subdir)
                if "{}jobs".format(os.path.sep) in fsubdir:
                    continue
                for root, dirs, files in salt.utils.path.os_walk(fsubdir):
                    for name in files:
                        if name.startswith("."):
                            continue
                        path = os.path.join(root, name)
                        try:
                            fmode = os.stat(path)
                        except OSError:
                            pass
                        if fmode.st_uid != uid or fmode.st_gid != gid:
                            if permissive and fmode.st_gid in groups:
                                pass
                            else:
                                # chown the file for the new user
                                os.chown(path, uid, gid)
                    for name in dirs:
                        path = os.path.join(root, name)
                        fmode = os.stat(path)
                        if fmode.st_uid != uid or fmode.st_gid != gid:
                            if permissive and fmode.st_gid in groups:
                                pass
                            else:
                                # chown the file for the new user
                                os.chown(path, uid, gid)
        # Allow the pki dir to be 700 or 750, but nothing else.
        # This prevents other users from writing out keys, while
        # allowing the use-case of 3rd-party software (like django)
        # to read in what it needs to integrate.
        #
        # If the permissions aren't correct, default to the more secure 700.
        # If acls are enabled, the pki_dir needs to remain readable, this
        # is still secure because the private keys are still only readable
        # by the user running the master
        if dir_ == pki_dir:
            smode = stat.S_IMODE(mode.st_mode)
            if smode != 448 and smode != 488:
                if os.access(dir_, os.W_OK):
                    os.chmod(dir_, 448)
                else:
                    msg = 'Unable to securely set the permissions of "{0}".'
                    msg = msg.format(dir_)
                    if is_console_configured():
                        log.critical(msg)
                    else:
                        sys.stderr.write("CRITICAL: {}\n".format(msg))

    if skip_extra is False:
        # Run the extra verification checks
        zmq_version()


def check_user(user):
    """
    Check user and assign process uid/gid.
    """
    if salt.utils.platform.is_windows():
        return True
    if user == salt.utils.user.get_user():
        return True

    # after confirming not running Windows
    pwuser = _get_pwnam(user)

    try:
        if hasattr(os, "initgroups"):
            os.initgroups(user, pwuser.pw_gid)  # pylint: disable=minimum-python-version
        else:
            os.setgroups(salt.utils.user.get_gid_list(user, include_default=False))
        os.setgid(pwuser.pw_gid)
        os.setuid(pwuser.pw_uid)

        # We could just reset the whole environment but let's just override
        # the variables we can get from pwuser
        if "HOME" in os.environ:
            os.environ["HOME"] = pwuser.pw_dir

        if "SHELL" in os.environ:
            os.environ["SHELL"] = pwuser.pw_shell

        for envvar in ("USER", "LOGNAME"):
            if envvar in os.environ:
                os.environ[envvar] = pwuser.pw_name

    except OSError:
        msg = 'Salt configured to run as user "{}" but unable to switch.'.format(user)
        if is_console_configured():
            log.critical(msg)
        else:
            sys.stderr.write("CRITICAL: {}\n".format(msg))
        return False
    return True


def list_path_traversal(path):
    """
    Returns a full list of directories leading up to, and including, a path.

    So list_path_traversal('/path/to/salt') would return:
        ['/', '/path', '/path/to', '/path/to/salt']
    in that order.

    This routine has been tested on Windows systems as well.
    list_path_traversal('c:\\path\\to\\salt') on Windows would return:
        ['c:\\', 'c:\\path', 'c:\\path\\to', 'c:\\path\\to\\salt']
    """
    out = [path]
    (head, tail) = os.path.split(path)
    if tail == "":
        # paths with trailing separators will return an empty string
        out = [head]
        (head, tail) = os.path.split(head)
    while head != out[0]:
        # loop until head is the same two consecutive times
        out.insert(0, head)
        (head, tail) = os.path.split(head)
    return out


def check_path_traversal(path, user="root", skip_perm_errors=False):
    """
    Walk from the root up to a directory and verify that the current
    user has access to read each directory. This is used for  making
    sure a user can read all parent directories of the minion's  key
    before trying to go and generate a new key and raising an IOError
    """
    for tpath in list_path_traversal(path):
        if not os.access(tpath, os.R_OK):
            msg = "Could not access {}.".format(tpath)
            if not os.path.exists(tpath):
                msg += " Path does not exist."
            else:
                current_user = salt.utils.user.get_user()
                # Make the error message more intelligent based on how
                # the user invokes salt-call or whatever other script.
                if user != current_user:
                    msg += " Try running as user {}.".format(user)
                else:
                    msg += " Please give {} read permissions.".format(user)

            # We don't need to bail on config file permission errors
            # if the CLI
            # process is run with the -a flag
            if skip_perm_errors:
                return
            # Propagate this exception up so there isn't a sys.exit()
            # in the middle of code that could be imported elsewhere.
            raise SaltClientError(msg)


def check_max_open_files(opts):
    """
    Check the number of max allowed open files and adjust if needed
    """
    mof_c = opts.get("max_open_files", 100000)
    if sys.platform.startswith("win"):
        # Check the Windows API for more detail on this
        # http://msdn.microsoft.com/en-us/library/xt874334(v=vs.71).aspx
        # and the python binding http://timgolden.me.uk/pywin32-docs/win32file.html
        mof_s = mof_h = win32file._getmaxstdio()
    else:
        mof_s, mof_h = resource.getrlimit(resource.RLIMIT_NOFILE)

    accepted_keys_dir = os.path.join(opts.get("pki_dir"), "minions")
    accepted_count = len(os.listdir(accepted_keys_dir))

    log.debug("This salt-master instance has accepted %s minion keys.", accepted_count)

    level = logging.INFO

    if (accepted_count * 4) <= mof_s:
        # We check for the soft value of max open files here because that's the
        # value the user chose to raise to.
        #
        # The number of accepted keys multiplied by four(4) is lower than the
        # soft value, everything should be OK
        return

    msg = (
        "The number of accepted minion keys({}) should be lower than 1/4 "
        "of the max open files soft setting({}). ".format(accepted_count, mof_s)
    )

    if accepted_count >= mof_s:
        # This should never occur, it might have already crashed
        msg += "salt-master will crash pretty soon! "
        level = logging.CRITICAL
    elif (accepted_count * 2) >= mof_s:
        # This is way too low, CRITICAL
        level = logging.CRITICAL
    elif (accepted_count * 3) >= mof_s:
        level = logging.WARNING
        # The accepted count is more than 3 time, WARN
    elif (accepted_count * 4) >= mof_s:
        level = logging.INFO

    if mof_c < mof_h:
        msg += (
            "According to the system's hard limit, there's still a "
            "margin of {} to raise the salt's max_open_files "
            "setting. ".format(mof_h - mof_c)
        )

    msg += "Please consider raising this value."
    log.log(level=level, msg=msg)


def _realpath_darwin(path):
    base = ""
    for part in path.split(os.path.sep)[1:]:
        if base != "":
            if os.path.islink(os.path.sep.join([base, part])):
                base = os.readlink(os.path.sep.join([base, part]))
            else:
                base = os.path.abspath(os.path.sep.join([base, part]))
        else:
            base = os.path.abspath(os.path.sep.join([base, part]))
    return base


def _realpath_windows(path):
    base = ""
    for part in path.split(os.path.sep):
        if base != "":
            try:
                # Need to use salt.utils.path.readlink as it handles junctions
                part = salt.utils.path.readlink(os.path.sep.join([base, part]))
                base = os.path.abspath(part)
            except OSError:
                base = os.path.abspath(os.path.sep.join([base, part]))
        else:
            base = part
    return base


def _realpath(path):
    """
    Cross platform realpath method. On Windows when python 3, this method
    uses the os.readlink method to resolve any filesystem links.
    All other platforms and version use ``os.path.realpath``.
    """
    if salt.utils.platform.is_darwin():
        return _realpath_darwin(path)
    elif salt.utils.platform.is_windows():
        return _realpath_windows(path)
    return os.path.realpath(path)


def clean_path(root, path, subdir=False):
    """
    Accepts the root the path needs to be under and verifies that the path is
    under said root. Pass in subdir=True if the path can result in a
    subdirectory of the root instead of having to reside directly in the root
    """
    real_root = _realpath(root)
    if not os.path.isabs(real_root):
        return ""
    if not os.path.isabs(path):
        path = os.path.join(root, path)
    path = os.path.normpath(path)
    real_path = _realpath(path)
    if subdir:
        if real_path.startswith(real_root):
            return real_path
    else:
        if os.path.dirname(real_path) == os.path.normpath(real_root):
            return real_path
    return ""


def valid_id(opts, id_):
    """
    Returns if the passed id is valid
    """
    try:
        if any(x in id_ for x in ("/", "\\", "\0")):
            return False
        return bool(clean_path(opts["pki_dir"], id_))
    except (AttributeError, KeyError, TypeError, UnicodeDecodeError):
        return False


def safe_py_code(code):
    """
    Check a string to see if it has any potentially unsafe routines which
    could be executed via python, this routine is used to improve the
    safety of modules suct as virtualenv
    """
    bads = ("import", ";", "subprocess", "eval", "open", "file", "exec", "input")
    for bad in bads:
        if code.count(bad):
            return False
    return True


def verify_log(opts):
    """
    If an insecre logging configuration is found, show a warning
    """
    level = LOG_LEVELS.get(str(opts.get("log_level")).lower(), logging.NOTSET)

    if level < logging.INFO:
        log.warning(
            "Insecure logging configuration detected! Sensitive data may be logged."
        )


def win_verify_env(path, dirs, permissive=False, pki_dir="", skip_extra=False):
    """
    Verify that the named directories are in place and that the environment
    can shake the salt
    """
    import salt.utils.win_functions
    import salt.utils.win_dacl
    import salt.utils.path

    # Make sure the file_roots is not set to something unsafe since permissions
    # on that directory are reset

    # `salt.utils.path.safe_path` will consider anything inside `C:\Windows` to
    # be unsafe. In some instances the test suite uses
    # `C:\Windows\Temp\salt-tests-tmpdir\rootdir` as the file_roots. So, we need
    # to consider anything in `C:\Windows\Temp` to be safe
    system_root = os.environ.get("SystemRoot", r"C:\Windows")
    allow_path = "\\".join([system_root, "TEMP"])
    if not salt.utils.path.safe_path(path=path, allow_path=allow_path):
        raise CommandExecutionError(
            "`file_roots` set to a possibly unsafe location: {}".format(path)
        )

    # Create the root path directory if missing
    if not os.path.isdir(path):
        os.makedirs(path)

    current_user = salt.utils.win_functions.get_current_user()
    # Set permissions to the registry key
    if salt.utils.win_functions.is_admin(current_user):
        reg_path = "HKLM\\SOFTWARE\\Salt Project\\salt"
        if not salt.utils.win_reg.key_exists(
            hive="HKLM", key="SOFTWARE\\Salt Project\\salt"
        ):
            salt.utils.win_reg.set_value(
                hive="HKLM", key="SOFTWARE\\Salt Project\\salt"
            )
        try:
            # Make the Administrators group owner
            # Use the SID to be locale agnostic
            salt.utils.win_dacl.set_owner(
                obj_name=reg_path, principal="S-1-5-32-544", obj_type="registry"
            )
        except CommandExecutionError:
            msg = 'Unable to securely set the owner of "{}".'.format(reg_path)
            if is_console_configured():
                log.critical(msg)
            else:
                sys.stderr.write("CRITICAL: {}\n".format(msg))

        try:
            # Get a clean dacl by not passing an obj_name
            dacl = salt.utils.win_dacl.dacl(obj_type="registry")

            # Add aces to the dacl, use the GUID (locale non-specific)
            # Administrators Group
            dacl.add_ace(
                principal="S-1-5-32-544",
                access_mode="grant",
                permissions="full_control",
                applies_to="this_key_subkeys",
            )
            # System
            dacl.add_ace(
                principal="S-1-5-18",
                access_mode="grant",
                permissions="full_control",
                applies_to="this_key_subkeys",
            )
            # Owner
            dacl.add_ace(
                principal="S-1-3-4",
                access_mode="grant",
                permissions="full_control",
                applies_to="this_key_subkeys",
            )

            # Save the dacl to the object
            dacl.save(obj_name=reg_path, protected=True)

        except CommandExecutionError:
            msg = 'Unable to securely set the permissions of "{}"'.format(reg_path)
            if is_console_configured():
                log.critical(msg)
            else:
                sys.stderr.write("CRITICAL: {}\n".format(msg))

    # Set permissions to the root path directory
    if salt.utils.win_functions.is_admin(current_user):
        try:
            # Make the Administrators group owner
            # Use the SID to be locale agnostic
            salt.utils.win_dacl.set_owner(obj_name=path, principal="S-1-5-32-544")

        except CommandExecutionError:
            msg = "Unable to securely set the owner of {}".format(path)
            if is_console_configured():
                log.critical(msg)
            else:
                sys.stderr.write("CRITICAL: {}\n".format(msg))

        if not permissive:
            try:
                # Get a clean dacl by not passing an obj_name
                dacl = salt.utils.win_dacl.dacl()

                # Add aces to the dacl, use the GUID (locale non-specific)
                # Administrators Group
                dacl.add_ace(
                    principal="S-1-5-32-544",
                    access_mode="grant",
                    permissions="full_control",
                    applies_to="this_folder_subfolders_files",
                )
                # System
                dacl.add_ace(
                    principal="S-1-5-18",
                    access_mode="grant",
                    permissions="full_control",
                    applies_to="this_folder_subfolders_files",
                )
                # Owner
                dacl.add_ace(
                    principal="S-1-3-4",
                    access_mode="grant",
                    permissions="full_control",
                    applies_to="this_folder_subfolders_files",
                )

                # Save the dacl to the object
                dacl.save(obj_name=path, protected=True)

            except CommandExecutionError:
                msg = 'Unable to securely set the permissions of "{}".'.format(path)
                if is_console_configured():
                    log.critical(msg)
                else:
                    sys.stderr.write("CRITICAL: {}\n".format(msg))

    # Create the directories
    for dir_ in dirs:
        if not dir_:
            continue
        if not os.path.isdir(dir_):
            try:
                os.makedirs(dir_)
            except OSError as err:
                msg = 'Failed to create directory path "{0}" - {1}\n'
                sys.stderr.write(msg.format(dir_, err))
                sys.exit(err.errno)

        # The PKI dir gets its own permissions
        if dir_ == pki_dir:
            try:
                # Make Administrators group the owner
                salt.utils.win_dacl.set_owner(obj_name=path, principal="S-1-5-32-544")

                # Give Admins, System and Owner permissions
                # Get a clean dacl by not passing an obj_name
                dacl = salt.utils.win_dacl.dacl()

                # Add aces to the dacl, use the GUID (locale non-specific)
                # Administrators Group
                dacl.add_ace(
                    principal="S-1-5-32-544",
                    access_mode="grant",
                    permissions="full_control",
                    applies_to="this_folder_subfolders_files",
                )
                # System
                dacl.add_ace(
                    principal="S-1-5-18",
                    access_mode="grant",
                    permissions="full_control",
                    applies_to="this_folder_subfolders_files",
                )
                # Owner
                dacl.add_ace(
                    principal="S-1-3-4",
                    access_mode="grant",
                    permissions="full_control",
                    applies_to="this_folder_subfolders_files",
                )

                # Save the dacl to the object
                dacl.save(obj_name=dir_, protected=True)

            except CommandExecutionError:
                msg = 'Unable to securely set the permissions of "{0}".'
                msg = msg.format(dir_)
                if is_console_configured():
                    log.critical(msg)
                else:
                    sys.stderr.write("CRITICAL: {}\n".format(msg))

    if skip_extra is False:
        # Run the extra verification checks
        zmq_version()



"""
Tests for the nacl execution module
"""

import salt.utils.stringutils
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf

try:
    import libnacl.secret  # pylint: disable=unused-import
    import libnacl.sealed  # pylint: disable=unused-import
    import salt.modules.nacl as nacl

    HAS_LIBNACL = True
except (ImportError, OSError, AttributeError):
    HAS_LIBNACL = False


@skipIf(not HAS_LIBNACL, "skipping test_nacl, libnacl is unavailable")
class NaclTest(TestCase, LoaderModuleMockMixin):
    """
    Test the nacl runner
    """

    def setup_loader_modules(self):
        self.unencrypted_data = salt.utils.stringutils.to_bytes("hello")
        self.opts = salt.config.DEFAULT_MINION_OPTS.copy()
        utils = salt.loader.utils(self.opts)
        funcs = salt.loader.minion_mods(self.opts, utils=utils, whitelist=["nacl"])

        return {
            nacl: {"__opts__": self.opts, "__utils__": utils, "__salt__": funcs},
        }

    def setUp(self):
        # Generate the keys
        ret = nacl.keygen()
        self.assertIn("pk", ret)
        self.assertIn("sk", ret)
        self.pk = ret["pk"]
        self.sk = ret["sk"]

    def test_keygen(self):
        """
        Test keygen
        """
        self.assertEqual(len(self.pk), 44)
        self.assertEqual(len(self.sk), 44)

    def test_enc_dec(self):
        """
        Generate keys, encrypt, then decrypt.
        """
        # Encrypt with pk
        encrypted_data = nacl.enc(data=self.unencrypted_data, pk=self.pk)

        # Decrypt with sk
        decrypted_data = nacl.dec(data=encrypted_data, sk=self.sk)
        self.assertEqual(self.unencrypted_data, decrypted_data)

    def test_sealedbox_enc_dec(self):
        """
        Generate keys, encrypt, then decrypt.
        """
        # Encrypt with pk
        encrypted_data = nacl.sealedbox_encrypt(data=self.unencrypted_data, pk=self.pk)

        # Decrypt with sk
        decrypted_data = nacl.sealedbox_decrypt(data=encrypted_data, sk=self.sk)

        self.assertEqual(self.unencrypted_data, decrypted_data)

    def test_secretbox_enc_dec(self):
        """
        Generate keys, encrypt, then decrypt.
        """
        # Encrypt with sk
        encrypted_data = nacl.secretbox_encrypt(data=self.unencrypted_data, sk=self.sk)

        # Decrypt with sk
        decrypted_data = nacl.secretbox_decrypt(data=encrypted_data, sk=self.sk)

        self.assertEqual(self.unencrypted_data, decrypted_data)



"""
JIRA Execution module
=====================

.. versionadded:: 2019.2.0

Execution module to manipulate JIRA tickets via Salt.

This module requires the ``jira`` Python library to be installed.

Configuration example:

.. code-block:: yaml

  jira:
    server: https://jira.atlassian.org
    username: salt
    password: pass
"""

import logging

import salt.utils.args

try:
    import jira

    HAS_JIRA = True
except ImportError:
    HAS_JIRA = False

log = logging.getLogger(__name__)

__virtualname__ = "jira"
__proxyenabled__ = ["*"]

JIRA = None


def __virtual__():
    return (
        __virtualname__
        if HAS_JIRA
        else (False, "Please install the jira Python libary from PyPI")
    )


def _get_credentials(server=None, username=None, password=None):
    """
    Returns the credentials merged with the config data (opts + pillar).
    """
    jira_cfg = __salt__["config.merge"]("jira", default={})
    if not server:
        server = jira_cfg.get("server")
    if not username:
        username = jira_cfg.get("username")
    if not password:
        password = jira_cfg.get("password")
    return server, username, password


def _get_jira(server=None, username=None, password=None):
    global JIRA
    if not JIRA:
        server, username, password = _get_credentials(
            server=server, username=username, password=password
        )
        JIRA = jira.JIRA(
            basic_auth=(username, password), server=server, logging=True
        )  # We want logging
    return JIRA


def create_issue(
    project,
    summary,
    description,
    template_engine="jinja",
    context=None,
    defaults=None,
    saltenv="base",
    issuetype="Bug",
    priority="Normal",
    labels=None,
    assignee=None,
    server=None,
    username=None,
    password=None,
    **kwargs
):
    """
    Create a JIRA issue using the named settings. Return the JIRA ticket ID.

    project
        The name of the project to attach the JIRA ticket to.

    summary
        The summary (title) of the JIRA ticket. When the ``template_engine``
        argument is set to a proper value of an existing Salt template engine
        (e.g., ``jinja``, ``mako``, etc.) it will render the ``summary`` before
        creating the ticket.

    description
        The full body description of the JIRA ticket. When the ``template_engine``
        argument is set to a proper value of an existing Salt template engine
        (e.g., ``jinja``, ``mako``, etc.) it will render the ``description`` before
        creating the ticket.

    template_engine: ``jinja``
        The name of the template engine to be used to render the values of the
        ``summary`` and ``description`` arguments. Default: ``jinja``.

    context: ``None``
        The context to pass when rendering the ``summary`` and ``description``.
        This argument is ignored when ``template_engine`` is set as ``None``

    defaults: ``None``
        Default values to pass to the Salt rendering pipeline for the
        ``summary`` and ``description`` arguments.
        This argument is ignored when ``template_engine`` is set as ``None``.

    saltenv: ``base``
        The Salt environment name (for the rendering system).

    issuetype: ``Bug``
        The type of the JIRA ticket. Default: ``Bug``.

    priority: ``Normal``
        The priority of the JIRA ticket. Default: ``Normal``.

    labels: ``None``
        A list of labels to add to the ticket.

    assignee: ``None``
        The name of the person to assign the ticket to.

    CLI Examples:

    .. code-block:: bash

        salt '*' jira.create_issue NET 'Ticket title' 'Ticket description'
        salt '*' jira.create_issue NET 'Issue on {{ opts.id }}' 'Error detected on {{ opts.id }}' template_engine=jinja
    """
    if template_engine:
        summary = __salt__["file.apply_template_on_contents"](
            summary,
            template=template_engine,
            context=context,
            defaults=defaults,
            saltenv=saltenv,
        )
        description = __salt__["file.apply_template_on_contents"](
            description,
            template=template_engine,
            context=context,
            defaults=defaults,
            saltenv=saltenv,
        )
    jira_ = _get_jira(server=server, username=username, password=password)
    if not labels:
        labels = []
    data = {
        "project": {"key": project},
        "summary": summary,
        "description": description,
        "issuetype": {"name": issuetype},
        "priority": {"name": priority},
        "labels": labels,
    }
    data.update(salt.utils.args.clean_kwargs(**kwargs))
    issue = jira_.create_issue(data)
    issue_key = str(issue)
    if assignee:
        assign_issue(issue_key, assignee)
    return issue_key


def assign_issue(issue_key, assignee, server=None, username=None, password=None):
    """
    Assign the issue to an existing user. Return ``True`` when the issue has
    been properly assigned.

    issue_key
        The JIRA ID of the ticket to manipulate.

    assignee
        The name of the user to assign the ticket to.

    CLI Example:

    .. code-block:: bash

        salt '*' jira.assign_issue NET-123 example_user
    """
    jira_ = _get_jira(server=server, username=username, password=password)
    assigned = jira_.assign_issue(issue_key, assignee)
    return assigned


def add_comment(
    issue_key,
    comment,
    visibility=None,
    is_internal=False,
    server=None,
    username=None,
    password=None,
):
    """
    Add a comment to an existing ticket. Return ``True`` when it successfully
    added the comment.

    issue_key
        The issue ID to add the comment to.

    comment
        The body of the comment to be added.

    visibility: ``None``
        A dictionary having two keys:

        - ``type``: is ``role`` (or ``group`` if the JIRA server has configured
          comment visibility for groups).
        - ``value``: the name of the role (or group) to which viewing of this
          comment will be restricted.

    is_internal: ``False``
        Whether a comment has to be marked as ``Internal`` in Jira Service Desk.

    CLI Example:

    .. code-block:: bash

        salt '*' jira.add_comment NE-123 'This is a comment'
    """
    jira_ = _get_jira(server=server, username=username, password=password)
    comm = jira_.add_comment(
        issue_key, comment, visibility=visibility, is_internal=is_internal
    )
    return True


def issue_closed(issue_key, server=None, username=None, password=None):
    """
    Check if the issue is closed.

    issue_key
        The JIRA iD of the ticket to close.

    Returns:

    - ``True``: the ticket exists and it is closed.
    - ``False``: the ticket exists and it has not been closed.
    - ``None``: the ticket does not exist.

    CLI Example:

    .. code-block:: bash

        salt '*' jira.issue_closed NE-123
    """
    if not issue_key:
        return None
    jira_ = _get_jira(server=server, username=username, password=password)
    try:
        ticket = jira_.issue(issue_key)
    except jira.exceptions.JIRAError:
        # Ticket not found
        return None
    return ticket.fields().status.name == "Closed"



"""
Tests for loop state(s)
"""

import pytest
import salt.states.loop
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class LoopTestCase(TestCase, LoaderModuleMockMixin):

    mock = MagicMock(return_value=True)
    func = "foo.bar"
    m_args = ["foo", "bar", "baz"]
    m_kwargs = {"hello": "world"}
    condition = "m_ret is True"
    period = 1
    timeout = 3

    def setup_loader_modules(self):
        return {
            salt.states.loop: {
                "__opts__": {"test": False},
                "__salt__": {self.func: self.mock},
            }
        }

    def setUp(self):
        self.mock.reset_mock()

    def test_until(self):
        ret = salt.states.loop.until(
            name=self.func,
            m_args=self.m_args,
            m_kwargs=self.m_kwargs,
            condition=self.condition,
            period=self.period,
            timeout=self.timeout,
        )
        assert ret["result"] is True
        self.mock.assert_called_once_with(*self.m_args, **self.m_kwargs)

    def test_until_without_args(self):
        ret = salt.states.loop.until(
            name=self.func,
            m_kwargs=self.m_kwargs,
            condition=self.condition,
            period=self.period,
            timeout=self.timeout,
        )
        assert ret["result"] is True
        self.mock.assert_called_once_with(**self.m_kwargs)

    def test_until_without_kwargs(self):
        ret = salt.states.loop.until(
            name=self.func,
            m_args=self.m_args,
            condition=self.condition,
            period=self.period,
            timeout=self.timeout,
        )
        assert ret["result"] is True
        self.mock.assert_called_once_with(*self.m_args)

    def test_until_without_args_or_kwargs(self):
        ret = salt.states.loop.until(
            name=self.func,
            condition=self.condition,
            period=self.period,
            timeout=self.timeout,
        )
        assert ret["result"] is True
        self.mock.assert_called_once_with()


class LoopTestCaseNoEval(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.loop
    """

    def setup_loader_modules(self):
        opts = salt.config.DEFAULT_MINION_OPTS.copy()
        utils = salt.loader.utils(opts)
        return {salt.states.loop: {"__opts__": opts, "__utils__": utils}}

    def test_test_mode(self):
        """
        Test response when test_mode is enabled.
        """
        with patch.dict(
            salt.states.loop.__salt__, {"foo.foo": True}  # pylint: disable=no-member
        ), patch.dict(
            salt.states.loop.__opts__, {"test": True}  # pylint: disable=no-member
        ):
            self.assertDictEqual(
                salt.states.loop.until(name="foo.foo", condition="m_ret"),
                {
                    "name": "foo.foo",
                    "result": None,
                    "changes": {},
                    "comment": "The execution module foo.foo will be run",
                },
            )
            self.assertDictEqual(
                salt.states.loop.until_no_eval(name="foo.foo", expected=2),
                {
                    "name": "foo.foo",
                    "result": None,
                    "changes": {},
                    "comment": 'Would have waited for "foo.foo" to produce "2".',
                },
            )

    @pytest.mark.slow_test
    def test_immediate_success(self):
        """
        Test for an immediate success.
        """
        with patch.dict(
            salt.states.loop.__salt__,
            {  # pylint: disable=no-member
                "foo.foo": lambda: 2,
                "foo.baz": lambda x, y: True,
            },
        ), patch.dict(
            salt.states.loop.__utils__,
            {"foo.baz": lambda x, y: True},  # pylint: disable=no-member
        ):
            self.assertDictEqual(
                salt.states.loop.until(name="foo.foo", condition="m_ret"),
                {
                    "name": "foo.foo",
                    "result": True,
                    "changes": {},
                    "comment": "Condition m_ret was met",
                },
            )
            # Using default compare_operator 'eq'
            self.assertDictEqual(
                salt.states.loop.until_no_eval(name="foo.foo", expected=2),
                {
                    "name": "foo.foo",
                    "result": True,
                    "changes": {},
                    "comment": "Call provided the expected results in 1 attempts",
                },
            )
            # Using compare_operator 'gt'
            self.assertDictEqual(
                salt.states.loop.until_no_eval(
                    name="foo.foo", expected=1, compare_operator="gt"  # Returns 2
                ),
                {
                    "name": "foo.foo",
                    "result": True,
                    "changes": {},
                    "comment": "Call provided the expected results in 1 attempts",
                },
            )
            # Using compare_operator 'ne'
            self.assertDictEqual(
                salt.states.loop.until_no_eval(
                    name="foo.foo", expected=3, compare_operator="ne"  # Returns 2
                ),
                {
                    "name": "foo.foo",
                    "result": True,
                    "changes": {},
                    "comment": "Call provided the expected results in 1 attempts",
                },
            )
            # Using __utils__['foo.baz'] as compare_operator
            self.assertDictEqual(
                salt.states.loop.until_no_eval(
                    name="foo.foo",
                    expected="anything, mocked compare_operator returns True anyway",
                    compare_operator="foo.baz",
                ),
                {
                    "name": "foo.foo",
                    "result": True,
                    "changes": {},
                    "comment": "Call provided the expected results in 1 attempts",
                },
            )
            # Using __salt__['foo.baz]' as compare_operator
            self.assertDictEqual(
                salt.states.loop.until_no_eval(
                    name="foo.foo",
                    expected="anything, mocked compare_operator returns True anyway",
                    compare_operator="foo.baz",
                ),
                {
                    "name": "foo.foo",
                    "result": True,
                    "changes": {},
                    "comment": "Call provided the expected results in 1 attempts",
                },
            )

    def test_immediate_failure(self):
        """
        Test for an immediate failure.
        Period and timeout will be set to 0.01 to assume one attempt.
        """
        with patch.dict(
            salt.states.loop.__salt__, {"foo.bar": lambda: False}
        ):  # pylint: disable=no-member
            self.assertDictEqual(
                salt.states.loop.until(
                    name="foo.bar", condition="m_ret", period=0.01, timeout=0.01
                ),
                {
                    "name": "foo.bar",
                    "result": False,
                    "changes": {},
                    "comment": "Timed out while waiting for condition m_ret",
                },
            )

            self.assertDictEqual(
                salt.states.loop.until_no_eval(
                    name="foo.bar", expected=True, period=0.01, timeout=0.01
                ),
                {
                    "name": "foo.bar",
                    "result": False,
                    "changes": {},
                    "comment": (
                        "Call did not produce the expected result after 1 attempts"
                    ),
                },
            )

    def test_eval_exceptions(self):
        """
        Test a couple of eval exceptions.
        """
        with patch.dict(
            salt.states.loop.__salt__, {"foo.bar": lambda: None}
        ):  # pylint: disable=no-member
            self.assertRaises(
                SyntaxError,
                salt.states.loop.until,
                name="foo.bar",
                condition='raise NameError("FOO")',
            )
            self.assertRaises(
                NameError, salt.states.loop.until, name="foo.bar", condition="foo"
            )

    def test_no_eval_exceptions(self):
        """
        Test exception handling in until_no_eval.
        """
        with patch.dict(
            salt.states.loop.__salt__,  # pylint: disable=no-member
            {"foo.bar": MagicMock(side_effect=KeyError("FOO"))},
        ):
            self.assertDictEqual(
                salt.states.loop.until_no_eval(name="foo.bar", expected=True),
                {
                    "name": "foo.bar",
                    "result": False,
                    "changes": {},
                    "comment": (
                        "Exception occurred while executing foo.bar: {}:{}".format(
                            type(KeyError()), "'FOO'"
                        )
                    ),
                },
            )

    def test_retried_success(self):
        """
        Test if the function does indeed return after a fixed amount of retries.

        Note: Do not merge these two tests in one with-block, as the side_effect
        iterator is shared.
        """
        with patch.dict(
            salt.states.loop.__salt__,  # pylint: disable=no-member
            {
                "foo.bar": MagicMock(side_effect=range(1, 7))
            },  # pylint: disable=incompatible-py3-code
        ):
            self.assertDictEqual(
                salt.states.loop.until(
                    name="foo.bar", condition="m_ret == 5", period=0, timeout=1
                ),
                {
                    "name": "foo.bar",
                    "result": True,
                    "changes": {},
                    "comment": "Condition m_ret == 5 was met",
                },
            )

        with patch.dict(
            salt.states.loop.__salt__,  # pylint: disable=no-member
            {
                "foo.bar": MagicMock(side_effect=range(1, 7))
            },  # pylint: disable=incompatible-py3-code
        ):
            self.assertDictEqual(
                salt.states.loop.until_no_eval(
                    name="foo.bar", expected=5, period=0, timeout=1
                ),
                {
                    "name": "foo.bar",
                    "result": True,
                    "changes": {},
                    "comment": "Call provided the expected results in 5 attempts",
                },
            )

    def test_retried_failure(self):
        """
        Test if the function fails after the designated timeout.
        """
        with patch.dict(
            salt.states.loop.__salt__,  # pylint: disable=no-member
            {
                "foo.bar": MagicMock(side_effect=range(1, 7))
            },  # pylint: disable=incompatible-py3-code
        ):
            self.assertDictEqual(
                salt.states.loop.until(
                    name="foo.bar", condition="m_ret == 5", period=0.01, timeout=0.03
                ),
                {
                    "name": "foo.bar",
                    "result": False,
                    "changes": {},
                    "comment": "Timed out while waiting for condition m_ret == 5",
                },
            )

        # period and timeout below has been increased to keep windows machines from
        # returning a lower number of attempts (because it's slower).
        with patch.dict(
            salt.states.loop.__salt__,  # pylint: disable=no-member
            {
                "foo.bar": MagicMock(side_effect=range(1, 7))
            },  # pylint: disable=incompatible-py3-code
        ):
            self.assertDictEqual(
                salt.states.loop.until_no_eval(
                    name="foo.bar", expected=5, period=0.2, timeout=0.5
                ),
                {
                    "name": "foo.bar",
                    "result": False,
                    "changes": {},
                    "comment": (
                        "Call did not produce the expected result after 3 attempts"
                    ),
                },
            )



"""
    :codeauthor: :email:`Christian McHugh <christian.mchugh@gmail.com>`
"""

import salt.modules.config as config
import salt.modules.zabbix as zabbix
from salt.exceptions import SaltException
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase

CONN_ARGS = {}
CONN_ARGS["url"] = "http://test.url"
CONN_ARGS["auth"] = "1234"

GETID_QUERY_RESULT_OK = [
    {"internal": "0", "flags": "0", "groupid": "11", "name": "Databases"}
]
GETID_QUERY_RESULT_BAD = [
    {"internal": "0", "flags": "0", "groupid": "11", "name": "Databases"},
    {"another": "object"},
]

DEFINED_PARAMS = {
    "name": "beta",
    "eventsource": 2,
    "status": 0,
    "filter": {
        "evaltype": 2,
        "conditions": [{"conditiontype": 24, "operator": 2, "value": "db"}],
    },
    "operations": [
        {"operationtype": 2},
        {
            "operationtype": 4,
            "opgroup": [
                {"groupid": {"query_object": "hostgroup", "query_name": "Databases"}}
            ],
        },
    ],
    "empty_list": [],
}

SUBSTITUTED_DEFINED_PARAMS = {
    "status": "0",
    "filter": {
        "evaltype": "2",
        "conditions": [{"operator": "2", "conditiontype": "24", "value": "db"}],
    },
    "eventsource": "2",
    "name": "beta",
    "operations": [
        {"operationtype": "2"},
        {"opgroup": [{"groupid": "11"}], "operationtype": "4"},
    ],
    "empty_list": [],
}

EXISTING_OBJECT_PARAMS = {
    "status": "0",
    "operations": [
        {
            "operationtype": "2",
            "esc_period": "0",
            "evaltype": "0",
            "opconditions": [],
            "esc_step_to": "1",
            "actionid": "23",
            "esc_step_from": "1",
            "operationid": "64",
        },
        {
            "operationtype": "4",
            "esc_period": "0",
            "evaltype": "0",
            "opconditions": [],
            "esc_step_to": "1",
            "actionid": "23",
            "esc_step_from": "1",
            "opgroup": [{"groupid": "11", "operationid": "65"}],
            "operationid": "65",
        },
    ],
    "def_shortdata": "",
    "name": "beta",
    "esc_period": "0",
    "def_longdata": "",
    "filter": {
        "formula": "",
        "evaltype": "2",
        "conditions": [
            {
                "operator": "2",
                "conditiontype": "24",
                "formulaid": "A",
                "value": "DIFFERENT VALUE HERE",
            }
        ],
        "eval_formula": "A",
    },
    "eventsource": "2",
    "actionid": "23",
    "r_shortdata": "",
    "r_longdata": "",
    "recovery_msg": "0",
    "empty_list": [{"dict_key": "dic_val"}],
}

DIFF_PARAMS_RESULT = {
    "filter": {
        "evaltype": "2",
        "conditions": [{"operator": "2", "conditiontype": "24", "value": "db"}],
    },
    "empty_list": [],
}

DIFF_PARAMS_RESULT_WITH_ROLLBACK = {
    "new": DIFF_PARAMS_RESULT,
    "old": {
        "filter": {
            "formula": "",
            "evaltype": "2",
            "conditions": [
                {
                    "operator": "2",
                    "conditiontype": "24",
                    "formulaid": "A",
                    "value": "DIFFERENT VALUE HERE",
                }
            ],
            "eval_formula": "A",
        },
        "empty_list": [{"dict_key": "dic_val"}],
    },
}


class ZabbixTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.zabbix
    """

    def setup_loader_modules(self):
        return {
            zabbix: {
                "__salt__": {
                    "cmd.which_bin": lambda _: "zabbix_server",
                    "config.get": config.get,
                }
            },
            config: {"__opts__": {}},
        }

    def test_get_object_id_by_params(self):
        """
        Test get_object_id function with expected result from API call
        """
        with patch(
            "salt.modules.zabbix.run_query",
            MagicMock(return_value=GETID_QUERY_RESULT_OK),
        ):
            self.assertEqual(
                zabbix.get_object_id_by_params("hostgroup", "Databases"), "11"
            )

    def test_get_obj_id_by_params_fail(self):
        """
        Test get_object_id function with unexpected result from API call
        """
        with patch(
            "salt.modules.zabbix.run_query",
            MagicMock(return_value=GETID_QUERY_RESULT_BAD),
        ):
            self.assertRaises(
                SaltException, zabbix.get_object_id_by_params, "hostgroup", "Databases"
            )

    def test_substitute_params(self):
        """
        Test proper parameter substitution for defined input
        """
        with patch(
            "salt.modules.zabbix.get_object_id_by_params", MagicMock(return_value="11")
        ):
            self.assertEqual(
                zabbix.substitute_params(DEFINED_PARAMS), SUBSTITUTED_DEFINED_PARAMS
            )

    def test_substitute_params_fail(self):
        """
        Test proper parameter substitution if there is needed parameter missing
        """
        self.assertRaises(
            SaltException,
            zabbix.substitute_params,
            {"groupid": {"query_object": "hostgroup"}},
        )

    def test_compare_params(self):
        """
        Test result comparison of two params structures
        """
        self.assertEqual(
            zabbix.compare_params(SUBSTITUTED_DEFINED_PARAMS, EXISTING_OBJECT_PARAMS),
            DIFF_PARAMS_RESULT,
        )

    def test_compare_params_rollback(self):
        """
        Test result comparison of two params structures with rollback return value option
        """
        self.assertEqual(
            zabbix.compare_params(
                SUBSTITUTED_DEFINED_PARAMS, EXISTING_OBJECT_PARAMS, True
            ),
            DIFF_PARAMS_RESULT_WITH_ROLLBACK,
        )

    def test_compare_params_fail(self):
        """
        Test result comparison of two params structures where some data type mismatch exists
        """
        self.assertRaises(
            SaltException, zabbix.compare_params, {"dict": "val"}, {"dict": ["list"]}
        )

    def test_apiinfo_version(self):
        """
        Test apiinfo_version
        """
        module_return = "3.4.5"
        query_return = {"jsonrpc": "2.0", "result": "3.4.5", "id": 1}

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(zabbix.apiinfo_version(**CONN_ARGS), module_return)

    def test__login_getting_nested_parameters_from_config(self):
        """
        Test get the connection data as nested parameters from config
        """
        query_return = {"jsonrpc": "2.0", "result": "3.4.5", "id": 1}
        fake_connection_data = {
            "zabbix": {
                "user": "testuser",
                "password": "password",
                "url": "http://fake_url/zabbix/api_jsonrpc.php",
            }
        }
        login_return = {
            "url": "http://fake_url/zabbix/api_jsonrpc.php",
            "auth": "3.4.5",
        }

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.dict(zabbix.__pillar__, fake_connection_data):
                self.assertEqual(zabbix._login(), login_return)

    def test__login_getting_flat_parameters_from_config(self):
        """
        Test get the connection data as flat parameters from config
        """
        query_return = {"jsonrpc": "2.0", "result": "3.4.5", "id": 1}
        fake_connection_data = {
            "zabbix.user": "testuser",
            "zabbix.password": "password",
            "zabbix.url": "http://fake_url/zabbix/api_jsonrpc.php",
        }
        login_return = {
            "url": "http://fake_url/zabbix/api_jsonrpc.php",
            "auth": "3.4.5",
        }

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.dict(zabbix.__pillar__, fake_connection_data):
                self.assertEqual(zabbix._login(), login_return)

    def test__login_getting_empty_parameters_from_config(self):
        """
        Test get the connection data from config with an empty response
        """
        query_return = {"jsonrpc": "2.0", "result": "3.4.5", "id": 1}
        fake_connection_data = {}

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.dict(zabbix.__pillar__, fake_connection_data):
                with self.assertRaises(SaltException) as login_exception:
                    ret = zabbix._login()
                    self.assertEqual(
                        login_exception.strerror,
                        "URL is probably not correct! ('user')",
                    )

    def test_get_mediatype(self):
        """
        query_submitted = { "params": {"filter": {"description": 10}, "output": "extend"},
        "id": 0, "auth": "251feb98e3c25b6b7fb984b6c7a79817", "method": "mediatype.get"}
        """

        module_return = [
            {
                "mediatypeid": "10",
                "type": "0",
                "name": "Testing",
                "smtp_server": "mail.example.com",
                "smtp_helo": "example.com",
                "smtp_email": "zabbix@example.com",
            }
        ]

        query_return = {
            "jsonrpc": "2.0",
            "result": [
                {
                    "mediatypeid": "10",
                    "type": "0",
                    "name": "Testing",
                    "smtp_server": "mail.example.com",
                    "smtp_helo": "example.com",
                    "smtp_email": "zabbix@example.com",
                }
            ],
            "id": 0,
        }
        zabbix_version_return_list = ["3.4", "4.4.5"]
        for zabbix_version_return in zabbix_version_return_list:
            patch_apiinfo_version = patch.object(
                zabbix,
                "apiinfo_version",
                autospec=True,
                return_value=zabbix_version_return,
            )
            patch_query = patch.object(
                zabbix, "_query", autospec=True, return_value=query_return
            )
            patch_login = patch.object(
                zabbix, "_login", autospec=True, return_value=CONN_ARGS
            )
            with patch_apiinfo_version, patch_query, patch_login:
                self.assertEqual(zabbix.mediatype_get("10", **CONN_ARGS), module_return)

    def test_user_create(self):
        """
        query_submitted = {"params": {"passwd": "password007", "alias": "james",
        "name": "James Bond", "usrgrps": [{"usrgrpid": 7}, {"usrgrpid": 12}]},
        "jsonrpc": "2.0", "id": 0, "auth": "f016981c4f0d3f8b9682e34588fe8a33",
        "method": "user.create"}
        """

        module_return = ["3"]
        query_return = {"jsonrpc": "2.0", "result": {"userids": ["3"]}, "id": 0}

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(
                    zabbix.user_create(
                        "james",
                        "password007",
                        "[7, 12]",
                        firstname="James Bond",
                        **CONN_ARGS
                    ),
                    module_return,
                )

    def test_user_delete(self):
        """
        query_submitted = {"params": [3], "jsonrpc": "2.0", "id": 0,
        "auth": "68d38eace8e42a35c8d0c6a2ab0245a6", "method": "user.delete"}
        """

        module_return = ["3"]
        query_return = {"jsonrpc": "2.0", "result": {"userids": ["3"]}, "id": 0}

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(zabbix.user_delete(3, **CONN_ARGS), module_return)

    def test_user_exists(self):
        """
        query_submitted = {"params": {"filter": {"alias": "Admin"}, "output": "extend"},
        "jsonrpc": "2.0", "id": 0, "auth": "72435c7f754cb2adb4ecddc98216057f",
        "method": "user.get"}
        """

        module_return = True
        # pylint: disable=E8128
        query_return = {
            "jsonrpc": "2.0",
            "result": [
                {
                    "userid": "1",
                    "alias": "Admin",
                    "name": "Zabbix",
                    "surname": "Administrator",
                    "url": "",
                    "autologin": "1",
                    "autologout": "0",
                    "lang": "en_GB",
                    "refresh": "30s",
                    "type": "3",
                    "theme": "default",
                    "attempt_failed": "0",
                    "attempt_ip": "10.0.2.2",
                    "attempt_clock": "1515922072",
                    "rows_per_page": "50",
                }
            ],
            "id": 0,
        }

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(
                    zabbix.user_exists("Admin", **CONN_ARGS), module_return
                )

    def test_user_get(self):
        """
        query_submitted = {"params": {"filter": {"alias": "Admin"}, "output": "extend"},
        "jsonrpc": "2.0", "id": 0, "auth": "49ef327f205d9e9150d4651cb6adc2d5",
        "method": "user.get"}
        """

        module_return = [
            {
                "lang": "en_GB",
                "rows_per_page": "50",
                "surname": "Administrator",
                "name": "Zabbix",
                "url": "",
                "attempt_clock": "1515922072",
                "userid": "1",
                "autologin": "1",
                "refresh": "30s",
                "attempt_failed": "0",
                "alias": "Admin",
                "theme": "default",
                "autologout": "0",
                "attempt_ip": "10.0.2.2",
                "type": "3",
            }
        ]
        # pylint: disable=E8128
        query_return = {
            "jsonrpc": "2.0",
            "result": [
                {
                    "userid": "1",
                    "alias": "Admin",
                    "name": "Zabbix",
                    "surname": "Administrator",
                    "url": "",
                    "autologin": "1",
                    "autologout": "0",
                    "lang": "en_GB",
                    "refresh": "30s",
                    "type": "3",
                    "theme": "default",
                    "attempt_failed": "0",
                    "attempt_ip": "10.0.2.2",
                    "attempt_clock": "1515922072",
                    "rows_per_page": "50",
                }
            ],
            "id": 0,
        }

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(zabbix.user_get("Admin", **CONN_ARGS), module_return)
                self.assertEqual(
                    zabbix.user_get(userids="1", **CONN_ARGS), module_return
                )

    def test_user_update(self):
        """
        query_submitted = {"params": {"userid": 3, "name": "James Brown"}, "jsonrpc": "2.0",
        "id": 0, "auth": "cdf2ee35e3bc47560585e9c457cbc398", "method": "user.update"}
        """

        module_return = ["3"]
        query_return = {"jsonrpc": "2.0", "result": {"userids": ["3"]}, "id": 0}

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(
                    zabbix.user_update("3", visible_name="James Brown", **CONN_ARGS),
                    module_return,
                )

    def test_user_getmedia(self):
        """
        query_submitted = {"params": {"userids": 3}, "jsonrpc": "2.0", "id": 0,
        "auth": "d4de741ea7cdd434b3ba7b56efa4efaf", "method": "usermedia.get"}
        """

        module_return = [
            {
                "mediatypeid": "1",
                "mediaid": "1",
                "severity": "63",
                "userid": "3",
                "period": "1-7,00:00-24:00",
                "sendto": "email@example.com",
                "active": "0",
            }
        ]
        # pylint: disable=E8128
        query_return = {
            "jsonrpc": "2.0",
            "result": [
                {
                    "mediaid": "1",
                    "userid": "3",
                    "mediatypeid": "1",
                    "sendto": "email@example.com",
                    "active": "0",
                    "severity": "63",
                    "period": "1-7,00:00-24:00",
                }
            ],
            "id": 0,
        }

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(zabbix.user_getmedia("3", **CONN_ARGS), module_return)

    def test_user_addmedia(self):
        """
        query_submitted = {"params": {"medias": [{"active": 0, "mediatypeid": 1,
        "period": "1-7,00:00-24:00", "severity": 63, "sendto": "support2@example.com"}],
        "users": [{"userid": 1}]}, "jsonrpc": "2.0", "id": 0, "auth": "b347fc1bf1f5617b93755619a037c19e",
        "method": "user.addmedia"}
        """

        module_return = ["2"]
        query_return = {"jsonrpc": "2.0", "result": {"mediaids": ["2"]}, "id": 0}

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(
                    zabbix.user_addmedia(
                        "1",
                        active="0",
                        mediatypeid="1",
                        period="1-7,00:00-24:00",
                        sendto="support2@example.com",
                        severity="63",
                        **CONN_ARGS
                    ),
                    module_return,
                )

    def test_user_deletemedia(self):
        """
        query_submitted = {"params": [1], "jsonrpc": "2.0", "id": 0, "auth": "9fb226c759a320de0de3b7a141404506",
        "method": "user.deletemedia"}
        """

        module_return = [1]
        query_return = {"jsonrpc": "2.0", "result": {"mediaids": [1]}, "id": 0}

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(
                    zabbix.user_deletemedia("1", **CONN_ARGS), module_return
                )

    def test_user_list(self):
        """
        query_submitted = {"params": {"output": "extend"}, "jsonrpc": "2.0", "id": 0,
        "auth": "54d67b63c37e690cf06972678f1e9720", "method": "user.get"}
        """

        module_return = [
            {
                "lang": "en_GB",
                "rows_per_page": "50",
                "surname": "Administrator",
                "name": "Zabbix",
                "url": "",
                "attempt_clock": "1515922072",
                "userid": "1",
                "autologin": "1",
                "refresh": "30s",
                "attempt_failed": "0",
                "alias": "Admin",
                "theme": "default",
                "autologout": "0",
                "attempt_ip": "10.0.2.2",
                "type": "3",
            },
            {
                "lang": "en_GB",
                "rows_per_page": "50",
                "surname": "",
                "name": "",
                "url": "",
                "attempt_clock": "0",
                "userid": "2",
                "autologin": "0",
                "refresh": "30s",
                "attempt_failed": "0",
                "alias": "guest",
                "theme": "default",
                "autologout": "15m",
                "attempt_ip": "",
                "type": "1",
            },
            {
                "lang": "en_GB",
                "rows_per_page": "50",
                "surname": "",
                "name": "James Brown",
                "url": "",
                "attempt_clock": "0",
                "userid": "5",
                "autologin": "0",
                "refresh": "30s",
                "attempt_failed": "0",
                "alias": "james",
                "theme": "default",
                "autologout": "15m",
                "attempt_ip": "",
                "type": "1",
            },
        ]
        # pylint: disable=E8128
        query_return = {
            "jsonrpc": "2.0",
            "result": [
                {
                    "userid": "1",
                    "alias": "Admin",
                    "name": "Zabbix",
                    "surname": "Administrator",
                    "url": "",
                    "autologin": "1",
                    "autologout": "0",
                    "lang": "en_GB",
                    "refresh": "30s",
                    "type": "3",
                    "theme": "default",
                    "attempt_failed": "0",
                    "attempt_ip": "10.0.2.2",
                    "attempt_clock": "1515922072",
                    "rows_per_page": "50",
                },
                {
                    "userid": "2",
                    "alias": "guest",
                    "name": "",
                    "surname": "",
                    "url": "",
                    "autologin": "0",
                    "autologout": "15m",
                    "lang": "en_GB",
                    "refresh": "30s",
                    "type": "1",
                    "theme": "default",
                    "attempt_failed": "0",
                    "attempt_ip": "",
                    "attempt_clock": "0",
                    "rows_per_page": "50",
                },
                {
                    "userid": "5",
                    "alias": "james",
                    "name": "James Brown",
                    "surname": "",
                    "url": "",
                    "autologin": "0",
                    "autologout": "15m",
                    "lang": "en_GB",
                    "refresh": "30s",
                    "type": "1",
                    "theme": "default",
                    "attempt_failed": "0",
                    "attempt_ip": "",
                    "attempt_clock": "0",
                    "rows_per_page": "50",
                },
            ],
            "id": 0,
        }

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(zabbix.user_list(**CONN_ARGS), module_return)

    def test_usergroup_create(self):
        """
        query_submitted = {"params": {"name": "testgroup"}, "jsonrpc": "2.0", "id": 0,
        "auth": "7f3ac5e90201e5de4eb19e5322606575", "method": "usergroup.create"}
        """

        module_return = ["13"]
        query_return = {"jsonrpc": "2.0", "result": {"usrgrpids": ["13"]}, "id": 0}

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(
                    zabbix.usergroup_create("testgroup", **CONN_ARGS), module_return
                )

    def test_usergroup_delete(self):
        """
        query_submitted = {"params": [13], "jsonrpc": "2.0", "id": 0,
        "auth": "9bad39de2a5a9211da588dd06dad8773", "method": "usergroup.delete"}
        """

        module_return = ["13"]
        query_return = {"jsonrpc": "2.0", "result": {"usrgrpids": ["13"]}, "id": 0}

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(
                    zabbix.usergroup_delete("13", **CONN_ARGS), module_return
                )

    def test_usergroup_exists(self):
        """
        query_submitted = {"params": {"filter": {"name": "testgroup"}, "output": "extend",
        "selectRights": "extend"}, "jsonrpc": "2.0", "id": 0, "auth": "e62424cd7aa71f6748e1d69c190ac852",
        "method": "usergroup.get"}
        """

        module_return = True
        query_return = {
            "jsonrpc": "2.0",
            "result": [
                {
                    "usrgrpid": "13",
                    "name": "testgroup",
                    "gui_access": "0",
                    "users_status": "0",
                    "debug_mode": "0",
                    "rights": [],
                }
            ],
            "id": 0,
        }

        with patch.object(zabbix, "apiinfo_version", return_value="3.2"):
            with patch.object(zabbix, "_query", return_value=query_return):
                with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                    self.assertEqual(
                        zabbix.usergroup_exists("testgroup", **CONN_ARGS), module_return
                    )

    def test_usergroup_get(self):
        """
        query_submitted = {"params": {"filter": {"name": "testgroup"}, "output": "extend",
        "selectRights": "extend"}, "jsonrpc": "2.0", "id": 0, "auth": "739cf358050f2a2d33162fdcfa714a3c",
        "method": "usergroup.get"}
        """

        module_return = [
            {
                "name": "testgroup",
                "rights": [],
                "users_status": "0",
                "gui_access": "0",
                "debug_mode": "0",
                "usrgrpid": "13",
            }
        ]
        query_return = {
            "jsonrpc": "2.0",
            "result": [
                {
                    "usrgrpid": "13",
                    "name": "testgroup",
                    "gui_access": "0",
                    "users_status": "0",
                    "debug_mode": "0",
                    "rights": [],
                }
            ],
            "id": 0,
        }

        with patch.object(zabbix, "apiinfo_version", return_value="3.2"):
            with patch.object(zabbix, "_query", return_value=query_return):
                with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                    self.assertEqual(
                        zabbix.usergroup_get("testgroup", **CONN_ARGS), module_return
                    )

    def test_usergroup_update(self):
        """
        query_submitted = {"params": {"usrgrpid": 13, "users_status": 1}, "jsonrpc": "2.0",
        "id": 0, "auth": "ef772237245f59f655871bc8fbbcd67c", "method": "usergroup.update"}
        """

        module_return = ["13"]
        query_return = {"jsonrpc": "2.0", "result": {"usrgrpids": ["13"]}, "id": 0}

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(
                    zabbix.usergroup_update("13", users_status="1", **CONN_ARGS),
                    module_return,
                )

    def test_usergroup_list(self):
        """
        query_submitted = {"params": {"output": "extend"}, "jsonrpc": "2.0", "id": 0,
        "auth": "4bc366bc7803c07e80f15b1bc14dc61f", "method": "usergroup.get"}
        """

        module_return = [
            {
                "usrgrpid": "7",
                "gui_access": "0",
                "debug_mode": "0",
                "name": "Zabbix administrators",
                "users_status": "0",
            },
            {
                "usrgrpid": "8",
                "gui_access": "0",
                "debug_mode": "0",
                "name": "Guests",
                "users_status": "0",
            },
            {
                "usrgrpid": "9",
                "gui_access": "0",
                "debug_mode": "0",
                "name": "Disabled",
                "users_status": "1",
            },
            {
                "usrgrpid": "11",
                "gui_access": "0",
                "debug_mode": "1",
                "name": "Enabled debug mode",
                "users_status": "0",
            },
            {
                "usrgrpid": "12",
                "gui_access": "2",
                "debug_mode": "0",
                "name": "No access to the frontend",
                "users_status": "0",
            },
            {
                "usrgrpid": "13",
                "gui_access": "0",
                "debug_mode": "0",
                "name": "testgroup",
                "users_status": "0",
            },
        ]
        # pylint: disable=E8128
        query_return = {
            "jsonrpc": "2.0",
            "result": [
                {
                    "usrgrpid": "7",
                    "name": "Zabbix administrators",
                    "gui_access": "0",
                    "users_status": "0",
                    "debug_mode": "0",
                },
                {
                    "usrgrpid": "8",
                    "name": "Guests",
                    "gui_access": "0",
                    "users_status": "0",
                    "debug_mode": "0",
                },
                {
                    "usrgrpid": "9",
                    "name": "Disabled",
                    "gui_access": "0",
                    "users_status": "1",
                    "debug_mode": "0",
                },
                {
                    "usrgrpid": "11",
                    "name": "Enabled debug mode",
                    "gui_access": "0",
                    "users_status": "0",
                    "debug_mode": "1",
                },
                {
                    "usrgrpid": "12",
                    "name": "No access to the frontend",
                    "gui_access": "2",
                    "users_status": "0",
                    "debug_mode": "0",
                },
                {
                    "usrgrpid": "13",
                    "name": "testgroup",
                    "gui_access": "0",
                    "users_status": "0",
                    "debug_mode": "0",
                },
            ],
            "id": 0,
        }

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(zabbix.usergroup_list(**CONN_ARGS), module_return)

    def test_host_inventory_get(self):
        """
        test host_inventory_get
        """
        module_return = {
            "poc_2_email": "",
            "poc_2_phone_b": "",
            "site_country": "",
            "poc_2_screen": "",
            "poc_2_notes": "",
            "poc_1_screen": "",
            "hardware": "",
            "software_app_a": "",
            "software_app_b": "",
            "software_app_c": "",
            "software_app_d": "",
            "os_short": "",
            "site_zip": "",
            "poc_2_name": "",
            "os_full": "",
            "host_netmask": "",
            "host_router": "",
            "url_c": "",
            "date_hw_install": "",
            "poc_1_phone_b": "",
            "poc_1_phone_a": "",
            "poc_1_cell": "",
            "type_full": "",
            "location_lat": "",
            "vendor": "",
            "contact": "",
            "site_rack": "",
            "location": "",
            "poc_2_cell": "",
            "date_hw_expiry": "",
            "installer_name": "",
            "type": "",
            "contract_number": "",
            "deployment_status": "",
            "site_notes": "",
            "inventory_mode": "0",
            "oob_ip": "",
            "host_networks": "",
            "hardware_full": "",
            "poc_2_phone_a": "",
            "poc_1_name": "",
            "site_state": "",
            "chassis": "",
            "software_app_e": "",
            "site_address_b": "",
            "site_address_a": "",
            "date_hw_decomm": "",
            "date_hw_purchase": "",
            "location_lon": "",
            "hw_arch": "",
            "software_full": "",
            "asset_tag": "",
            "oob_router": "",
            "hostid": "10258",
            "poc_1_email": "",
            "name": "",
            "poc_1_notes": "",
            "serialno_b": "",
            "notes": "",
            "oob_netmask": "",
            "alias": "other thing",
            "tag": "",
            "macaddress_b": "",
            "macaddress_a": "",
            "site_city": "",
            "site_address_c": "",
            "model": "",
            "serialno_a": "",
            "os": "some",
            "url_b": "",
            "url_a": "",
            "software": "",
        }
        query_return = {
            "jsonrpc": "2.0",
            "result": [
                {
                    "hostid": "10258",
                    "proxy_hostid": "0",
                    "host": "master",
                    "status": "0",
                    "disable_until": "1517766661",
                    "error": (
                        "Get value from agent failed: cannot connect to"
                        " [[10.0.2.15]:10050]: [111] Connection refused"
                    ),
                    "available": "2",
                    "errors_from": "1516087871",
                    "lastaccess": "0",
                    "ipmi_authtype": "-1",
                    "ipmi_privilege": "2",
                    "ipmi_username": "",
                    "ipmi_password": "",
                    "ipmi_disable_until": "0",
                    "ipmi_available": "0",
                    "snmp_disable_until": "0",
                    "snmp_available": "0",
                    "maintenanceid": "0",
                    "maintenance_status": "0",
                    "maintenance_type": "0",
                    "maintenance_from": "0",
                    "ipmi_errors_from": "0",
                    "snmp_errors_from": "0",
                    "ipmi_error": "",
                    "snmp_error": "",
                    "jmx_disable_until": "0",
                    "jmx_available": "0",
                    "jmx_errors_from": "0",
                    "jmx_error": "",
                    "name": "master",
                    "flags": "0",
                    "templateid": "0",
                    "description": "",
                    "tls_connect": "1",
                    "tls_accept": "1",
                    "tls_issuer": "",
                    "tls_subject": "",
                    "tls_psk_identity": "",
                    "tls_psk": "",
                    "inventory": {
                        "hostid": "10258",
                        "inventory_mode": "0",
                        "type": "",
                        "type_full": "",
                        "name": "",
                        "alias": "other thing",
                        "os": "some",
                        "os_full": "",
                        "os_short": "",
                        "serialno_a": "",
                        "serialno_b": "",
                        "tag": "",
                        "asset_tag": "",
                        "macaddress_a": "",
                        "macaddress_b": "",
                        "hardware": "",
                        "hardware_full": "",
                        "software": "",
                        "software_full": "",
                        "software_app_a": "",
                        "software_app_b": "",
                        "software_app_c": "",
                        "software_app_d": "",
                        "software_app_e": "",
                        "contact": "",
                        "location": "",
                        "location_lat": "",
                        "location_lon": "",
                        "notes": "",
                        "chassis": "",
                        "model": "",
                        "hw_arch": "",
                        "vendor": "",
                        "contract_number": "",
                        "installer_name": "",
                        "deployment_status": "",
                        "url_a": "",
                        "url_b": "",
                        "url_c": "",
                        "host_networks": "",
                        "host_netmask": "",
                        "host_router": "",
                        "oob_ip": "",
                        "oob_netmask": "",
                        "oob_router": "",
                        "date_hw_purchase": "",
                        "date_hw_install": "",
                        "date_hw_expiry": "",
                        "date_hw_decomm": "",
                        "site_address_a": "",
                        "site_address_b": "",
                        "site_address_c": "",
                        "site_city": "",
                        "site_state": "",
                        "site_country": "",
                        "site_zip": "",
                        "site_rack": "",
                        "site_notes": "",
                        "poc_1_name": "",
                        "poc_1_email": "",
                        "poc_1_phone_a": "",
                        "poc_1_phone_b": "",
                        "poc_1_cell": "",
                        "poc_1_screen": "",
                        "poc_1_notes": "",
                        "poc_2_name": "",
                        "poc_2_email": "",
                        "poc_2_phone_a": "",
                        "poc_2_phone_b": "",
                        "poc_2_cell": "",
                        "poc_2_screen": "",
                        "poc_2_notes": "",
                    },
                }
            ],
            "id": 1,
        }

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(
                    zabbix.host_inventory_get("12345", **CONN_ARGS), module_return
                )

    def test_host_inventory_get_with_disabled_inventory(self):
        """
        test host_inventory_get with a host with inventory disabled
        """
        module_return = False
        query_return = {
            "jsonrpc": "2.0",
            "result": [
                {
                    "hostid": "10258",
                    "proxy_hostid": "0",
                    "host": "master",
                    "status": "0",
                    "disable_until": "1517766661",
                    "error": "Get value from agent failed: cannot connect to [[10.0.2.15]:10050]: [111] Connection refused",
                    "available": "2",
                    "errors_from": "1516087871",
                    "lastaccess": "0",
                    "ipmi_authtype": "-1",
                    "ipmi_privilege": "2",
                    "ipmi_username": "",
                    "ipmi_password": "",
                    "ipmi_disable_until": "0",
                    "ipmi_available": "0",
                    "snmp_disable_until": "0",
                    "snmp_available": "0",
                    "maintenanceid": "0",
                    "maintenance_status": "0",
                    "maintenance_type": "0",
                    "maintenance_from": "0",
                    "ipmi_errors_from": "0",
                    "snmp_errors_from": "0",
                    "ipmi_error": "",
                    "snmp_error": "",
                    "jmx_disable_until": "0",
                    "jmx_available": "0",
                    "jmx_errors_from": "0",
                    "jmx_error": "",
                    "name": "master",
                    "flags": "0",
                    "templateid": "0",
                    "description": "",
                    "tls_connect": "1",
                    "tls_accept": "1",
                    "tls_issuer": "",
                    "tls_subject": "",
                    "tls_psk_identity": "",
                    "tls_psk": "",
                    "inventory": [],
                }
            ],
            "id": 1,
        }

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(
                    zabbix.host_inventory_get("12345", **CONN_ARGS), module_return
                )

    def test_host_inventory_get_with_a_missing_host(self):
        """
        test host_inventory_get with a non-existent host
        """
        module_return = False
        query_return = {
            "jsonrpc": "2.0",
            "result": [],
            "id": 0,
        }

        with patch.object(zabbix, "_query", return_value=query_return):
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(
                    zabbix.host_inventory_get("12345", **CONN_ARGS), module_return
                )

    def test_host_inventory_set(self):
        """
        query_submitted = {"params": {"hostid": 10258, "inventory_mode": "0", "inventory":
        {"asset_tag": "jml3322", "type": "Xen"}}, "jsonrpc": "2.0", "id": 0,
        "auth": "a50d2c3030b9b73d7c28b5ebd89c044c", "method": "host.update"}
        """

        module_return = {"hostids": [10258]}
        query_return = {"jsonrpc": "2.0", "result": {"hostids": [10258]}, "id": 0}
        with patch.object(
            zabbix, "_query", autospec=True, return_value=query_return
        ) as mock__query:
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(
                    zabbix.host_inventory_set(
                        10258, asset_tag="jml3322", type="Xen", **CONN_ARGS
                    ),
                    module_return,
                )
                mock__query.assert_called_with(
                    "host.update",
                    {
                        "hostid": 10258,
                        "inventory_mode": "0",
                        "inventory": {
                            "asset_tag": "jml3322",
                            "type": "Xen",
                            "url": "http://test.url",
                            "auth": "1234",
                        },
                    },
                    "http://test.url",
                    "1234",
                )

    def test_host_inventory_set_with_inventory_mode(self):
        """
        query_submitted = {"params": {"hostid": 10258, "inventory_mode": "1", "inventory":
        {"asset_tag": "jml3322", "type": "Xen"}}, "jsonrpc": "2.0", "id": 0,
        "auth": "a50d2c3030b9b73d7c28b5ebd89c044c", "method": "host.update"}
        """

        module_return = {"hostids": [10258]}
        query_return = {"jsonrpc": "2.0", "result": {"hostids": [10258]}, "id": 0}
        with patch.object(
            zabbix, "_query", autospec=True, return_value=query_return
        ) as mock__query:
            with patch.object(zabbix, "_login", return_value=CONN_ARGS):
                self.assertEqual(
                    zabbix.host_inventory_set(
                        10258,
                        asset_tag="jml3322",
                        type="Xen",
                        inventory_mode="1",
                        **CONN_ARGS
                    ),
                    module_return,
                )
                mock__query.assert_called_with(
                    "host.update",
                    {
                        "hostid": 10258,
                        "inventory_mode": "1",
                        "inventory": {
                            "asset_tag": "jml3322",
                            "type": "Xen",
                            "url": "http://test.url",
                            "auth": "1234",
                        },
                    },
                    "http://test.url",
                    "1234",
                )



import codecs
import glob
import logging
import os
import textwrap

import salt.loader
import salt.utils.data
import salt.utils.files
import salt.utils.reactor as reactor
import salt.utils.yaml
from tests.support.mixins import AdaptedConfigurationTestCaseMixin
from tests.support.mock import MagicMock, Mock, mock_open, patch
from tests.support.unit import TestCase

REACTOR_CONFIG = """\
reactor:
  - old_runner:
    - /srv/reactor/old_runner.sls
  - old_wheel:
    - /srv/reactor/old_wheel.sls
  - old_local:
    - /srv/reactor/old_local.sls
  - old_cmd:
    - /srv/reactor/old_cmd.sls
  - old_caller:
    - /srv/reactor/old_caller.sls
  - new_runner:
    - /srv/reactor/new_runner.sls
  - new_wheel:
    - /srv/reactor/new_wheel.sls
  - new_local:
    - /srv/reactor/new_local.sls
  - new_cmd:
    - /srv/reactor/new_cmd.sls
  - new_caller:
    - /srv/reactor/new_caller.sls
"""

REACTOR_DATA = {
    "runner": {"data": {"message": "This is an error"}},
    "wheel": {"data": {"id": "foo"}},
    "local": {"data": {"pkg": "zsh", "repo": "updates"}},
    "cmd": {"data": {"pkg": "zsh", "repo": "updates"}},
    "caller": {"data": {"path": "/tmp/foo"}},
}

SLS = {
    "/srv/reactor/old_runner.sls": textwrap.dedent(
        """\
        raise_error:
          runner.error.error:
            - name: Exception
            - message: {{ data['data']['message'] }}
        """
    ),
    "/srv/reactor/old_wheel.sls": textwrap.dedent(
        """\
        remove_key:
          wheel.key.delete:
            - match: {{ data['data']['id'] }}
        """
    ),
    "/srv/reactor/old_local.sls": textwrap.dedent(
        """\
        install_zsh:
          local.state.single:
            - tgt: test
            - arg:
              - pkg.installed
              - {{ data['data']['pkg'] }}
            - kwarg:
                fromrepo: {{ data['data']['repo'] }}
        """
    ),
    "/srv/reactor/old_cmd.sls": textwrap.dedent(
        """\
        install_zsh:
          cmd.state.single:
            - tgt: test
            - arg:
              - pkg.installed
              - {{ data['data']['pkg'] }}
            - kwarg:
                fromrepo: {{ data['data']['repo'] }}
        """
    ),
    "/srv/reactor/old_caller.sls": textwrap.dedent(
        """\
        touch_file:
          caller.file.touch:
            - args:
              - {{ data['data']['path'] }}
        """
    ),
    "/srv/reactor/new_runner.sls": textwrap.dedent(
        """\
        raise_error:
          runner.error.error:
            - args:
              - name: Exception
              - message: {{ data['data']['message'] }}
        """
    ),
    "/srv/reactor/new_wheel.sls": textwrap.dedent(
        """\
        remove_key:
          wheel.key.delete:
            - args:
              - match: {{ data['data']['id'] }}
        """
    ),
    "/srv/reactor/new_local.sls": textwrap.dedent(
        """\
        install_zsh:
          local.state.single:
            - tgt: test
            - args:
              - fun: pkg.installed
              - name: {{ data['data']['pkg'] }}
              - fromrepo: {{ data['data']['repo'] }}
        """
    ),
    "/srv/reactor/new_cmd.sls": textwrap.dedent(
        """\
        install_zsh:
          cmd.state.single:
            - tgt: test
            - args:
              - fun: pkg.installed
              - name: {{ data['data']['pkg'] }}
              - fromrepo: {{ data['data']['repo'] }}
        """
    ),
    "/srv/reactor/new_caller.sls": textwrap.dedent(
        """\
        touch_file:
          caller.file.touch:
            - args:
              - name: {{ data['data']['path'] }}
        """
    ),
}

LOW_CHUNKS = {
    # Note that the "name" value in the chunk has been overwritten by the
    # "name" argument in the SLS. This is one reason why the new schema was
    # needed.
    "old_runner": [
        {
            "state": "runner",
            "__id__": "raise_error",
            "__sls__": "/srv/reactor/old_runner.sls",
            "order": 1,
            "fun": "error.error",
            "name": "Exception",
            "message": "This is an error",
        }
    ],
    "old_wheel": [
        {
            "state": "wheel",
            "__id__": "remove_key",
            "name": "remove_key",
            "__sls__": "/srv/reactor/old_wheel.sls",
            "order": 1,
            "fun": "key.delete",
            "match": "foo",
        }
    ],
    "old_local": [
        {
            "state": "local",
            "__id__": "install_zsh",
            "name": "install_zsh",
            "__sls__": "/srv/reactor/old_local.sls",
            "order": 1,
            "tgt": "test",
            "fun": "state.single",
            "arg": ["pkg.installed", "zsh"],
            "kwarg": {"fromrepo": "updates"},
        }
    ],
    "old_cmd": [
        {
            "state": "local",  # 'cmd' should be aliased to 'local'
            "__id__": "install_zsh",
            "name": "install_zsh",
            "__sls__": "/srv/reactor/old_cmd.sls",
            "order": 1,
            "tgt": "test",
            "fun": "state.single",
            "arg": ["pkg.installed", "zsh"],
            "kwarg": {"fromrepo": "updates"},
        }
    ],
    "old_caller": [
        {
            "state": "caller",
            "__id__": "touch_file",
            "name": "touch_file",
            "__sls__": "/srv/reactor/old_caller.sls",
            "order": 1,
            "fun": "file.touch",
            "args": ["/tmp/foo"],
        }
    ],
    "new_runner": [
        {
            "state": "runner",
            "__id__": "raise_error",
            "name": "raise_error",
            "__sls__": "/srv/reactor/new_runner.sls",
            "order": 1,
            "fun": "error.error",
            "args": [{"name": "Exception"}, {"message": "This is an error"}],
        }
    ],
    "new_wheel": [
        {
            "state": "wheel",
            "__id__": "remove_key",
            "name": "remove_key",
            "__sls__": "/srv/reactor/new_wheel.sls",
            "order": 1,
            "fun": "key.delete",
            "args": [{"match": "foo"}],
        }
    ],
    "new_local": [
        {
            "state": "local",
            "__id__": "install_zsh",
            "name": "install_zsh",
            "__sls__": "/srv/reactor/new_local.sls",
            "order": 1,
            "tgt": "test",
            "fun": "state.single",
            "args": [
                {"fun": "pkg.installed"},
                {"name": "zsh"},
                {"fromrepo": "updates"},
            ],
        }
    ],
    "new_cmd": [
        {
            "state": "local",
            "__id__": "install_zsh",
            "name": "install_zsh",
            "__sls__": "/srv/reactor/new_cmd.sls",
            "order": 1,
            "tgt": "test",
            "fun": "state.single",
            "args": [
                {"fun": "pkg.installed"},
                {"name": "zsh"},
                {"fromrepo": "updates"},
            ],
        }
    ],
    "new_caller": [
        {
            "state": "caller",
            "__id__": "touch_file",
            "name": "touch_file",
            "__sls__": "/srv/reactor/new_caller.sls",
            "order": 1,
            "fun": "file.touch",
            "args": [{"name": "/tmp/foo"}],
        }
    ],
}

WRAPPER_CALLS = {
    "old_runner": (
        "error.error",
        {
            "__state__": "runner",
            "__id__": "raise_error",
            "__sls__": "/srv/reactor/old_runner.sls",
            "__user__": "Reactor",
            "order": 1,
            "arg": [],
            "kwarg": {"name": "Exception", "message": "This is an error"},
            "name": "Exception",
            "message": "This is an error",
        },
    ),
    "old_wheel": (
        "key.delete",
        {
            "__state__": "wheel",
            "__id__": "remove_key",
            "name": "remove_key",
            "__sls__": "/srv/reactor/old_wheel.sls",
            "order": 1,
            "__user__": "Reactor",
            "arg": ["foo"],
            "kwarg": {},
            "match": "foo",
        },
    ),
    "old_local": {
        "args": ("test", "state.single"),
        "kwargs": {
            "state": "local",
            "__id__": "install_zsh",
            "name": "install_zsh",
            "__sls__": "/srv/reactor/old_local.sls",
            "order": 1,
            "arg": ["pkg.installed", "zsh"],
            "kwarg": {"fromrepo": "updates"},
        },
    },
    "old_cmd": {
        "args": ("test", "state.single"),
        "kwargs": {
            "state": "local",
            "__id__": "install_zsh",
            "name": "install_zsh",
            "__sls__": "/srv/reactor/old_cmd.sls",
            "order": 1,
            "arg": ["pkg.installed", "zsh"],
            "kwarg": {"fromrepo": "updates"},
        },
    },
    "old_caller": {"args": ("file.touch", "/tmp/foo"), "kwargs": {}},
    "new_runner": (
        "error.error",
        {
            "__state__": "runner",
            "__id__": "raise_error",
            "name": "raise_error",
            "__sls__": "/srv/reactor/new_runner.sls",
            "__user__": "Reactor",
            "order": 1,
            "arg": (),
            "kwarg": {"name": "Exception", "message": "This is an error"},
        },
    ),
    "new_wheel": (
        "key.delete",
        {
            "__state__": "wheel",
            "__id__": "remove_key",
            "name": "remove_key",
            "__sls__": "/srv/reactor/new_wheel.sls",
            "order": 1,
            "__user__": "Reactor",
            "arg": (),
            "kwarg": {"match": "foo"},
        },
    ),
    "new_local": {
        "args": ("test", "state.single"),
        "kwargs": {
            "state": "local",
            "__id__": "install_zsh",
            "name": "install_zsh",
            "__sls__": "/srv/reactor/new_local.sls",
            "order": 1,
            "arg": (),
            "kwarg": {"fun": "pkg.installed", "name": "zsh", "fromrepo": "updates"},
        },
    },
    "new_cmd": {
        "args": ("test", "state.single"),
        "kwargs": {
            "state": "local",
            "__id__": "install_zsh",
            "name": "install_zsh",
            "__sls__": "/srv/reactor/new_cmd.sls",
            "order": 1,
            "arg": (),
            "kwarg": {"fun": "pkg.installed", "name": "zsh", "fromrepo": "updates"},
        },
    },
    "new_caller": {"args": ("file.touch",), "kwargs": {"name": "/tmp/foo"}},
}

log = logging.getLogger(__name__)


class TestReactor(TestCase, AdaptedConfigurationTestCaseMixin):
    """
    Tests for constructing the low chunks to be executed via the Reactor
    """

    @classmethod
    def setUpClass(cls):
        """
        Load the reactor config for mocking
        """
        cls.opts = cls.get_temp_config("master")
        reactor_config = salt.utils.yaml.safe_load(REACTOR_CONFIG)
        cls.opts.update(reactor_config)
        cls.reactor = reactor.Reactor(cls.opts)
        cls.reaction_map = salt.utils.data.repack_dictlist(reactor_config["reactor"])
        renderers = salt.loader.render(cls.opts, {})
        cls.render_pipe = [(renderers[x], "") for x in ("jinja", "yaml")]

    @classmethod
    def tearDownClass(cls):
        del cls.opts
        del cls.reactor
        del cls.render_pipe

    def test_list_reactors(self):
        """
        Ensure that list_reactors() returns the correct list of reactor SLS
        files for each tag.
        """
        for schema in ("old", "new"):
            for rtype in REACTOR_DATA:
                tag = "_".join((schema, rtype))
                self.assertEqual(
                    self.reactor.list_reactors(tag), self.reaction_map[tag]
                )

    def test_reactions(self):
        """
        Ensure that the correct reactions are built from the configured SLS
        files and tag data.
        """
        for schema in ("old", "new"):
            for rtype in REACTOR_DATA:
                tag = "_".join((schema, rtype))
                log.debug("test_reactions: processing %s", tag)
                reactors = self.reactor.list_reactors(tag)
                log.debug("test_reactions: %s reactors: %s", tag, reactors)
                # No globbing in our example SLS, and the files don't actually
                # exist, so mock glob.glob to just return back the path passed
                # to it.
                with patch.object(glob, "glob", MagicMock(side_effect=lambda x: [x])):
                    # The below four mocks are all so that
                    # salt.template.compile_template() will read the templates
                    # we've mocked up in the SLS global variable above.
                    with patch.object(os.path, "isfile", MagicMock(return_value=True)):
                        with patch.object(
                            salt.utils.files, "is_empty", MagicMock(return_value=False)
                        ):
                            with patch.object(
                                codecs, "open", mock_open(read_data=SLS[reactors[0]])
                            ):
                                with patch.object(
                                    salt.template,
                                    "template_shebang",
                                    MagicMock(return_value=self.render_pipe),
                                ):
                                    reactions = self.reactor.reactions(
                                        tag,
                                        REACTOR_DATA[rtype],
                                        reactors,
                                    )
                                    log.debug(
                                        "test_reactions: %s reactions: %s",
                                        tag,
                                        reactions,
                                    )
                                    self.assertEqual(reactions, LOW_CHUNKS[tag])


class TestReactWrap(TestCase, AdaptedConfigurationTestCaseMixin):
    """
    Tests that we are formulating the wrapper calls properly
    """

    @classmethod
    def setUpClass(cls):
        cls.wrap = reactor.ReactWrap(cls.get_temp_config("master"))

    @classmethod
    def tearDownClass(cls):
        del cls.wrap

    def test_runner(self):
        """
        Test runner reactions using both the old and new config schema
        """
        for schema in ("old", "new"):
            tag = "_".join((schema, "runner"))
            chunk = LOW_CHUNKS[tag][0]
            thread_pool = Mock()
            thread_pool.fire_async = Mock()
            with patch.object(self.wrap, "pool", thread_pool):
                self.wrap.run(chunk)
            thread_pool.fire_async.assert_called_with(
                self.wrap.client_cache["runner"].low, args=WRAPPER_CALLS[tag]
            )

    def test_wheel(self):
        """
        Test wheel reactions using both the old and new config schema
        """
        for schema in ("old", "new"):
            tag = "_".join((schema, "wheel"))
            chunk = LOW_CHUNKS[tag][0]
            thread_pool = Mock()
            thread_pool.fire_async = Mock()
            with patch.object(self.wrap, "pool", thread_pool):
                self.wrap.run(chunk)
            thread_pool.fire_async.assert_called_with(
                self.wrap.client_cache["wheel"].low, args=WRAPPER_CALLS[tag]
            )

    def test_local(self):
        """
        Test local reactions using both the old and new config schema
        """
        for schema in ("old", "new"):
            tag = "_".join((schema, "local"))
            chunk = LOW_CHUNKS[tag][0]
            client_cache = {"local": Mock()}
            client_cache["local"].cmd_async = Mock()
            with patch.object(self.wrap, "client_cache", client_cache):
                self.wrap.run(chunk)
            client_cache["local"].cmd_async.assert_called_with(
                *WRAPPER_CALLS[tag]["args"], **WRAPPER_CALLS[tag]["kwargs"]
            )

    def test_cmd(self):
        """
        Test cmd reactions (alias for 'local') using both the old and new
        config schema
        """
        for schema in ("old", "new"):
            tag = "_".join((schema, "cmd"))
            chunk = LOW_CHUNKS[tag][0]
            client_cache = {"local": Mock()}
            client_cache["local"].cmd_async = Mock()
            with patch.object(self.wrap, "client_cache", client_cache):
                self.wrap.run(chunk)
            client_cache["local"].cmd_async.assert_called_with(
                *WRAPPER_CALLS[tag]["args"], **WRAPPER_CALLS[tag]["kwargs"]
            )

    def test_caller(self):
        """
        Test caller reactions using both the old and new config schema
        """
        for schema in ("old", "new"):
            tag = "_".join((schema, "caller"))
            chunk = LOW_CHUNKS[tag][0]
            client_cache = {"caller": Mock()}
            client_cache["caller"].cmd = Mock()
            with patch.object(self.wrap, "client_cache", client_cache):
                self.wrap.run(chunk)
            client_cache["caller"].cmd.assert_called_with(
                *WRAPPER_CALLS[tag]["args"], **WRAPPER_CALLS[tag]["kwargs"]
            )



"""
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
"""

import os

import pytest
import salt.utils.files
from salt.exceptions import CommandExecutionError
from tests.support.case import ModuleCase
from tests.support.helpers import random_string, runs_on

# Create user strings for tests
ADD_USER = random_string("RS-", lowercase=False)
DEL_USER = random_string("RS-", lowercase=False)
PRIMARY_GROUP_USER = random_string("RS-", lowercase=False)
CHANGE_USER = random_string("RS-", lowercase=False)


@pytest.mark.skip_if_not_root
@runs_on(kernel="Darwin")
@pytest.mark.destructive_test
class MacUserModuleTest(ModuleCase):
    """
    Integration tests for the mac_user module
    """

    def setUp(self):
        """
        Sets up test requirements
        """
        super().setUp()
        os_grain = self.run_function("grains.item", ["kernel"])
        if os_grain["kernel"] not in "Darwin":
            self.skipTest("Test not applicable to '{kernel}' kernel".format(**os_grain))

    @pytest.mark.slow_test
    def test_mac_user_add(self):
        """
        Tests the add function
        """
        try:
            self.run_function("user.add", [ADD_USER])
            user_info = self.run_function("user.info", [ADD_USER])
            self.assertEqual(ADD_USER, user_info["name"])
        except CommandExecutionError:
            self.run_function("user.delete", [ADD_USER])
            raise

    @pytest.mark.slow_test
    def test_mac_user_delete(self):
        """
        Tests the delete function
        """

        # Create a user to delete - If unsuccessful, skip the test
        if self.run_function("user.add", [DEL_USER]) is not True:
            self.run_function("user.delete", [DEL_USER])
            self.skipTest("Failed to create a user to delete")

        # Now try to delete the added user
        ret = self.run_function("user.delete", [DEL_USER])
        self.assertTrue(ret)

    @pytest.mark.slow_test
    def test_mac_user_primary_group(self):
        """
        Tests the primary_group function
        """

        # Create a user to test primary group function
        if self.run_function("user.add", [PRIMARY_GROUP_USER]) is not True:
            self.run_function("user.delete", [PRIMARY_GROUP_USER])
            self.skipTest("Failed to create a user")

        try:
            # Test mac_user.primary_group
            primary_group = self.run_function(
                "user.primary_group", [PRIMARY_GROUP_USER]
            )
            uid_info = self.run_function("user.info", [PRIMARY_GROUP_USER])
            self.assertIn(primary_group, uid_info["groups"])

        except AssertionError:
            self.run_function("user.delete", [PRIMARY_GROUP_USER])
            raise

    @pytest.mark.slow_test
    def test_mac_user_changes(self):
        """
        Tests mac_user functions that change user properties
        """
        # Create a user to manipulate - if unsuccessful, skip the test
        if self.run_function("user.add", [CHANGE_USER]) is not True:
            self.run_function("user.delete", [CHANGE_USER])
            self.skipTest("Failed to create a user")

        try:
            # Test mac_user.chuid
            self.run_function("user.chuid", [CHANGE_USER, 4376])
            uid_info = self.run_function("user.info", [CHANGE_USER])
            self.assertEqual(uid_info["uid"], 4376)

            # Test mac_user.chgid
            self.run_function("user.chgid", [CHANGE_USER, 4376])
            gid_info = self.run_function("user.info", [CHANGE_USER])
            self.assertEqual(gid_info["gid"], 4376)

            # Test mac.user.chshell
            self.run_function("user.chshell", [CHANGE_USER, "/bin/zsh"])
            shell_info = self.run_function("user.info", [CHANGE_USER])
            self.assertEqual(shell_info["shell"], "/bin/zsh")

            # Test mac_user.chhome
            self.run_function("user.chhome", [CHANGE_USER, "/Users/foo"])
            home_info = self.run_function("user.info", [CHANGE_USER])
            self.assertEqual(home_info["home"], "/Users/foo")

            # Test mac_user.chfullname
            self.run_function("user.chfullname", [CHANGE_USER, "Foo Bar"])
            fullname_info = self.run_function("user.info", [CHANGE_USER])
            self.assertEqual(fullname_info["fullname"], "Foo Bar")

            # Test mac_user.chgroups
            self.run_function("user.chgroups", [CHANGE_USER, "wheel"])
            groups_info = self.run_function("user.info", [CHANGE_USER])
            self.assertEqual(groups_info["groups"], ["wheel"])

        except AssertionError:
            self.run_function("user.delete", [CHANGE_USER])
            raise

    @pytest.mark.slow_test
    def test_mac_user_enable_auto_login(self):
        """
        Tests mac_user functions that enable auto login
        """
        # Make sure auto login is disabled before we start
        if self.run_function("user.get_auto_login"):
            self.skipTest("Auto login already enabled")

        try:
            # Does enable return True
            self.assertTrue(
                self.run_function(
                    "user.enable_auto_login", ["Spongebob", "Squarepants"]
                )
            )

            # Did it set the user entry in the plist file
            self.assertEqual(self.run_function("user.get_auto_login"), "Spongebob")

            # Did it generate the `/etc/kcpassword` file
            self.assertTrue(os.path.exists("/etc/kcpassword"))

            # Are the contents of the file correct
            test_data = b".\xc3\xb8'B\xc2\xa0\xc3\x99\xc2\xad\xc2\x8b\xc3\x8d\xc3\x8dl"
            with salt.utils.files.fopen("/etc/kcpassword", "rb") as f:
                file_data = f.read()
            self.assertEqual(test_data, file_data)

            # Does disable return True
            self.assertTrue(self.run_function("user.disable_auto_login"))

            # Does it remove the user entry in the plist file
            self.assertFalse(self.run_function("user.get_auto_login"))

            # Is the `/etc/kcpassword` file removed
            self.assertFalse(os.path.exists("/etc/kcpassword"))

        finally:
            # Make sure auto_login is disabled
            self.assertTrue(self.run_function("user.disable_auto_login"))

            # Make sure autologin is disabled
            if self.run_function("user.get_auto_login"):
                raise Exception("Failed to disable auto login")

    @pytest.mark.slow_test
    def test_mac_user_disable_auto_login(self):
        """
        Tests mac_user functions that disable auto login
        """
        # Make sure auto login is enabled before we start
        # Is there an existing setting
        if self.run_function("user.get_auto_login"):
            self.skipTest("Auto login already enabled")

        try:
            # Enable auto login for the test
            self.run_function("user.enable_auto_login", ["Spongebob", "Squarepants"])

            # Make sure auto login got set up
            if not self.run_function("user.get_auto_login") == "Spongebob":
                raise Exception("Failed to enable auto login")

            # Does disable return True
            self.assertTrue(self.run_function("user.disable_auto_login"))

            # Does it remove the user entry in the plist file
            self.assertFalse(self.run_function("user.get_auto_login"))

            # Is the `/etc/kcpassword` file removed
            self.assertFalse(os.path.exists("/etc/kcpassword"))

        finally:
            # Make sure auto login is disabled
            self.assertTrue(self.run_function("user.disable_auto_login"))

            # Make sure auto login is disabled
            if self.run_function("user.get_auto_login"):
                raise Exception("Failed to disable auto login")

    def tearDown(self):
        """
        Clean up after tests
        """

        # Delete ADD_USER
        add_info = self.run_function("user.info", [ADD_USER])
        if add_info:
            self.run_function("user.delete", [ADD_USER])

        # Delete DEL_USER if something failed
        del_info = self.run_function("user.info", [DEL_USER])
        if del_info:
            self.run_function("user.delete", [DEL_USER])

        # Delete CHANGE_USER
        change_info = self.run_function("user.info", [CHANGE_USER])
        if change_info:
            self.run_function("user.delete", [CHANGE_USER])



import sys

import pytest
import salt.utils.data


def test_get_value_simple_path():
    data = {"a": {"b": {"c": "foo"}}}
    assert [{"value": "foo"}] == salt.utils.data.get_value(data, "a:b:c")


@pytest.mark.skipif(
    sys.version_info < (3, 6),
    reason="Test will randomly fail since Python3.5 does not have ordered dictionaries",
)
def test_get_value_placeholder_dict():
    data = {"a": {"b": {"name": "foo"}, "c": {"name": "bar"}}}
    assert [
        {"value": "foo", "id": "b"},
        {"value": "bar", "id": "c"},
    ] == salt.utils.data.get_value(data, "a:{id}:name")


@pytest.mark.skipif(
    sys.version_info < (3, 6),
    reason="Test will randomly fail since Python3.5 does not have ordered dictionaries",
)
def test_get_value_placeholder_list():
    data = {"a": [{"name": "foo"}, {"name": "bar"}]}
    assert [
        {"value": "foo", "id": 0},
        {"value": "bar", "id": 1},
    ] == salt.utils.data.get_value(data, "a:{id}:name")


@pytest.mark.skipif(
    sys.version_info < (3, 6),
    reason="Test will randomly fail since Python3.5 does not have ordered dictionaries",
)
def test_get_value_nested_placeholder():
    data = {
        "a": {
            "b": {"b1": {"name": "foo1"}, "b2": {"name": "foo2"}},
            "c": {"c1": {"name": "bar"}},
        }
    }
    assert [
        {"value": "foo1", "id": "b", "sub": "b1"},
        {"value": "foo2", "id": "b", "sub": "b2"},
        {"value": "bar", "id": "c", "sub": "c1"},
    ] == salt.utils.data.get_value(data, "a:{id}:{sub}:name")


def test_get_value_nested_notfound():
    data = {"a": {"b": {"c": "foo"}}}
    assert [{"value": []}] == salt.utils.data.get_value(data, "a:b:d", [])


def test_get_value_not_found():
    assert [{"value": []}] == salt.utils.data.get_value({}, "a", [])


def test_get_value_none():
    assert [{"value": None}] == salt.utils.data.get_value({"a": None}, "a")


def test_get_value_simple_type_path():
    assert [{"value": []}] == salt.utils.data.get_value({"a": 1024}, "a:b", [])


def test_get_value_None_path():
    assert [{"value": None}] == salt.utils.data.get_value({"a": None}, "a:b", [])



"""
Test case for the slack utils module
"""

import logging

import pytest
import salt.utils.slack as slack
from tests.support.mock import MagicMock, patch

log = logging.getLogger(__name__)


@pytest.fixture
def configure_loader_modules():
    yield {
        slack: {
            "__opts__": {
                "vault": {
                    "url": "http://127.0.0.1",
                    "auth": {
                        "token": "test",
                        "method": "token",
                        "uses": 15,
                        "ttl": 500,
                    },
                },
            },
        }
    }


def test_query():
    """
    Test case for the query function in the slack utils module
    """
    function = "message"
    api_key = "xoxp-xxxxxxxxxx-xxxxxxxxxx-xxxxxxxxxx-xxxxxx"
    args = None
    method = "POST"
    header_dict = {"Content-Type": "application/x-www-form-urlen coded"}
    data = "channel=%23general&username=Slack+User&as_user=Slack+User&text=%60%60%60id%3A+minion%0D%0Afunction%3A+test.ping%0D%0Afunction+args%3A+%5B%5D%0D%0Ajid%3A+20201017004822956482%0D%0Areturn%3A+true%0A%0D%0A%60%60%60"
    opts = None

    mock_result = {
        "body": '{"ok": false, "error": "token_revoked"}',
        "status": 200,
        "dict": {"ok": False, "error": "token_revoked"},
    }
    mock = MagicMock(return_value=mock_result)
    with patch("salt.utils.http.query", mock):
        expected = {"message": "token_revoked", "res": False}
        ret = slack.query(function, api_key, args, method, header_dict, data, opts)
        assert ret == expected

    mock_result = {
        "status": 0,
        "error": (
            "[SSL: CERTIFICATE_VERIFY_FAILED] certificate verify failed: unable to get"
            " local issuer certificate (_ssl.c:1076)"
        ),
    }
    mock = MagicMock(return_value=mock_result)
    with patch("salt.utils.http.query", mock):
        expected = {"message": "invalid_auth", "res": False}
        ret = slack.query(function, api_key, args, method, header_dict, data, opts)
        assert ret == expected

    mock_result = {"status": 0, "dict": {}}
    mock = MagicMock(return_value=mock_result)
    with patch("salt.utils.http.query", mock):
        expected = {"message": "Unknown response", "res": False}
        ret = slack.query(function, api_key, args, method, header_dict, data, opts)
        assert ret == expected



"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""


import salt.modules.groupadd as groupadd
import salt.utils.platform
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

try:
    import grp
except ImportError:
    pass


@skipIf(salt.utils.platform.is_windows(), "Module not available on Windows")
class GroupAddTestCase(TestCase, LoaderModuleMockMixin):
    """
    TestCase for salt.modules.groupadd
    """

    def setup_loader_modules(self):
        return {groupadd: {}}

    # 'add' function tests: 1

    def test_add(self):
        """
        Tests if specified group was added
        """
        mock = MagicMock(return_value={"retcode": 0})
        with patch.dict(groupadd.__salt__, {"cmd.run_all": mock}):
            self.assertTrue(groupadd.add("test", 100))

        with patch.dict(groupadd.__grains__, {"kernel": "Linux"}):
            with patch.dict(groupadd.__salt__, {"cmd.run_all": mock}):
                self.assertTrue(groupadd.add("test", 100, True))

    # 'info' function tests: 1

    def test_info(self):
        """
        Tests the return of group information
        """
        getgrnam = grp.struct_group(("foo", "*", 20, ["test"]))
        with patch("grp.getgrnam", MagicMock(return_value=getgrnam)):
            ret = {"passwd": "*", "gid": 20, "name": "foo", "members": ["test"]}
            self.assertEqual(groupadd.info("foo"), ret)

    # '_format_info' function tests: 1

    def test_format_info(self):
        """
        Tests the formatting of returned group information
        """
        group = {"passwd": "*", "gid": 0, "name": "test", "members": ["root"]}
        with patch("salt.modules.groupadd._format_info", MagicMock(return_value=group)):
            data = grp.struct_group(("wheel", "*", 0, ["root"]))
            ret = {"passwd": "*", "gid": 0, "name": "test", "members": ["root"]}
            self.assertDictEqual(groupadd._format_info(data), ret)

    # 'getent' function tests: 1

    def test_getent(self):
        """
        Tests the return of information on all groups
        """
        getgrnam = grp.struct_group(("foo", "*", 20, ["test"]))
        with patch("grp.getgrall", MagicMock(return_value=[getgrnam])):
            ret = [{"passwd": "*", "gid": 20, "name": "foo", "members": ["test"]}]
            self.assertEqual(groupadd.getent(), ret)

    # 'chgid' function tests: 2

    def test_chgid_gid_same(self):
        """
        Tests if the group id is the same as argument
        """
        mock = MagicMock(return_value={"gid": 10})
        with patch.object(groupadd, "info", mock):
            self.assertTrue(groupadd.chgid("test", 10))

    def test_chgid(self):
        """
        Tests the gid for a named group was changed
        """
        mock = MagicMock(return_value=None)
        with patch.dict(groupadd.__salt__, {"cmd.run": mock}):
            mock = MagicMock(side_effect=[{"gid": 10}, {"gid": 500}])
            with patch.object(groupadd, "info", mock):
                self.assertTrue(groupadd.chgid("test", 500))

    # 'delete' function tests: 1

    def test_delete(self):
        """
        Tests if the specified group was deleted
        """
        mock_ret = MagicMock(return_value={"retcode": 0})
        with patch.dict(groupadd.__salt__, {"cmd.run_all": mock_ret}):
            self.assertTrue(groupadd.delete("test"))

    # 'adduser' function tests: 1

    def test_adduser(self):
        """
        Tests if specified user gets added in the group.
        """
        os_version_list = [
            {
                "grains": {
                    "kernel": "Linux",
                    "os_family": "RedHat",
                    "osmajorrelease": "5",
                },
                "cmd": ["gpasswd", "-a", "root", "test"],
            },
            {
                "grains": {
                    "kernel": "Linux",
                    "os_family": "Suse",
                    "osmajorrelease": "11",
                },
                "cmd": ["usermod", "-A", "test", "root"],
            },
            {
                "grains": {"kernel": "Linux"},
                "cmd": ["gpasswd", "--add", "root", "test"],
            },
            {
                "grains": {"kernel": "OTHERKERNEL"},
                "cmd": ["usermod", "-G", "test", "root"],
            },
        ]

        for os_version in os_version_list:
            mock = MagicMock(return_value={"retcode": 0})
            with patch.dict(groupadd.__grains__, os_version["grains"]):
                with patch.dict(groupadd.__salt__, {"cmd.retcode": mock}):
                    self.assertFalse(groupadd.adduser("test", "root"))
                    groupadd.__salt__["cmd.retcode"].assert_called_once_with(
                        os_version["cmd"], python_shell=False
                    )

    # 'deluser' function tests: 1

    def test_deluser(self):
        """
        Tests if specified user gets deleted from the group.
        """
        os_version_list = [
            {
                "grains": {
                    "kernel": "Linux",
                    "os_family": "RedHat",
                    "osmajorrelease": "5",
                },
                "cmd": ["gpasswd", "-d", "root", "test"],
            },
            {
                "grains": {
                    "kernel": "Linux",
                    "os_family": "Suse",
                    "osmajorrelease": "11",
                },
                "cmd": ["usermod", "-R", "test", "root"],
            },
            {
                "grains": {"kernel": "Linux"},
                "cmd": ["gpasswd", "--del", "root", "test"],
            },
            {"grains": {"kernel": "OpenBSD"}, "cmd": ["usermod", "-S", "foo", "root"]},
        ]

        for os_version in os_version_list:
            mock_retcode = MagicMock(return_value=0)
            mock_stdout = MagicMock(return_value="test foo")
            mock_info = MagicMock(
                return_value={
                    "passwd": "*",
                    "gid": 0,
                    "name": "test",
                    "members": ["root"],
                }
            )

            with patch.dict(groupadd.__grains__, os_version["grains"]):
                with patch.dict(
                    groupadd.__salt__,
                    {
                        "cmd.retcode": mock_retcode,
                        "group.info": mock_info,
                        "cmd.run_stdout": mock_stdout,
                    },
                ):
                    self.assertTrue(groupadd.deluser("test", "root"))
                    groupadd.__salt__["cmd.retcode"].assert_called_once_with(
                        os_version["cmd"], python_shell=False
                    )

    # 'deluser' function tests: 1

    def test_members(self):
        """
        Tests if members of the group, get replaced with a provided list.
        """
        os_version_list = [
            {
                "grains": {
                    "kernel": "Linux",
                    "os_family": "RedHat",
                    "osmajorrelease": "5",
                },
                "cmd": ["gpasswd", "-M", "foo", "test"],
            },
            {
                "grains": {
                    "kernel": "Linux",
                    "os_family": "Suse",
                    "osmajorrelease": "11",
                },
                "cmd": ["groupmod", "-A", "foo", "test"],
            },
            {
                "grains": {"kernel": "Linux"},
                "cmd": ["gpasswd", "--members", "foo", "test"],
            },
            {"grains": {"kernel": "OpenBSD"}, "cmd": ["usermod", "-G", "test", "foo"]},
        ]

        for os_version in os_version_list:
            mock_ret = MagicMock(return_value={"retcode": 0})
            mock_stdout = MagicMock(return_value={"cmd.run_stdout": 1})
            mock_info = MagicMock(
                return_value={
                    "passwd": "*",
                    "gid": 0,
                    "name": "test",
                    "members": ["root"],
                }
            )
            mock = MagicMock(return_value=True)

            with patch.dict(groupadd.__grains__, os_version["grains"]):
                with patch.dict(
                    groupadd.__salt__,
                    {
                        "cmd.retcode": mock_ret,
                        "group.info": mock_info,
                        "cmd.run_stdout": mock_stdout,
                        "cmd.run": mock,
                    },
                ):
                    self.assertFalse(groupadd.members("test", "foo"))
                    groupadd.__salt__["cmd.retcode"].assert_called_once_with(
                        os_version["cmd"], python_shell=False
                    )



"""
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
"""

from salt.cli.batch import Batch
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class BatchTestCase(TestCase):
    """
    Unit Tests for the salt.cli.batch module
    """

    def setUp(self):
        opts = {
            "batch": "",
            "conf_file": {},
            "tgt": "",
            "transport": "",
            "timeout": 5,
            "gather_job_timeout": 5,
        }

        mock_client = MagicMock()
        with patch("salt.client.get_local_client", MagicMock(return_value=mock_client)):
            with patch("salt.client.LocalClient.cmd_iter", MagicMock(return_value=[])):
                self.batch = Batch(opts, quiet="quiet")

    # get_bnum tests

    def test_get_bnum_str(self):
        """
        Tests passing batch value as a number(str)
        """
        self.batch.opts = {"batch": "2", "timeout": 5}
        self.batch.minions = ["foo", "bar"]
        self.assertEqual(Batch.get_bnum(self.batch), 2)

    def test_get_bnum_int(self):
        """
        Tests passing batch value as a number(int)
        """
        self.batch.opts = {"batch": 2, "timeout": 5}
        self.batch.minions = ["foo", "bar"]
        self.assertEqual(Batch.get_bnum(self.batch), 2)

    def test_get_bnum_percentage(self):
        """
        Tests passing batch value as percentage
        """
        self.batch.opts = {"batch": "50%", "timeout": 5}
        self.batch.minions = ["foo"]
        self.assertEqual(Batch.get_bnum(self.batch), 1)

    def test_get_bnum_high_percentage(self):
        """
        Tests passing batch value as percentage over 100%
        """
        self.batch.opts = {"batch": "160%", "timeout": 5}
        self.batch.minions = ["foo", "bar", "baz"]
        self.assertEqual(Batch.get_bnum(self.batch), 4)

    def test_get_bnum_invalid_batch_data(self):
        """
        Tests when an invalid batch value is passed
        """
        ret = Batch.get_bnum(self.batch)
        self.assertEqual(ret, None)

    def test_return_value_in_run_for_ret(self):
        """
        cmd_iter_no_block should have been called with a return no matter if
        the return value was in ret or return.
        """
        self.batch.opts = {
            "batch": "100%",
            "timeout": 5,
            "fun": "test",
            "arg": "foo",
            "gather_job_timeout": 5,
            "ret": "my_return",
        }
        self.batch.gather_minions = MagicMock(
            return_value=[["foo", "bar", "baz"], [], []],
        )
        self.batch.local.cmd_iter_no_block = MagicMock(return_value=iter([]))
        ret = Batch.run(self.batch)
        # We need to fetch at least one object to trigger the relevant code path.
        x = next(ret)
        self.batch.local.cmd_iter_no_block.assert_called_with(
            ["baz", "bar", "foo"],
            "test",
            "foo",
            5,
            "list",
            raw=False,
            ret="my_return",
            show_jid=False,
            verbose=False,
            gather_job_timeout=5,
        )

    def test_return_value_in_run_for_return(self):
        """
        cmd_iter_no_block should have been called with a return no matter if
        the return value was in ret or return.
        """
        self.batch.opts = {
            "batch": "100%",
            "timeout": 5,
            "fun": "test",
            "arg": "foo",
            "gather_job_timeout": 5,
            "return": "my_return",
        }
        self.batch.gather_minions = MagicMock(
            return_value=[["foo", "bar", "baz"], [], []],
        )
        self.batch.local.cmd_iter_no_block = MagicMock(return_value=iter([]))
        ret = Batch.run(self.batch)
        # We need to fetch at least one object to trigger the relevant code path.
        x = next(ret)
        self.batch.local.cmd_iter_no_block.assert_called_with(
            ["baz", "bar", "foo"],
            "test",
            "foo",
            5,
            "list",
            raw=False,
            ret="my_return",
            show_jid=False,
            verbose=False,
            gather_job_timeout=5,
        )



"""
integration tests for mac_system
"""

import logging

import pytest
from tests.support.case import ModuleCase
from tests.support.helpers import random_string
from tests.support.unit import skipIf

log = logging.getLogger(__name__)


SET_COMPUTER_NAME = random_string("RS-", lowercase=False)
SET_SUBNET_NAME = random_string("RS-", lowercase=False)


@pytest.mark.flaky(max_runs=10)
@pytest.mark.skip_unless_on_darwin
@pytest.mark.usefixtures("salt_sub_minion")
@pytest.mark.skip_if_not_root
@pytest.mark.skip_if_binaries_missing("systemsetup")
class MacSystemModuleTest(ModuleCase):
    """
    Validate the mac_system module
    """

    ATRUN_ENABLED = False
    REMOTE_LOGIN_ENABLED = False
    REMOTE_EVENTS_ENABLED = False
    SUBNET_NAME = ""
    KEYBOARD_DISABLED = False

    def setUp(self):
        """
        Get current settings
        """
        self.ATRUN_ENABLED = self.run_function("service.enabled", ["com.apple.atrun"])
        self.REMOTE_LOGIN_ENABLED = self.run_function("system.get_remote_login")
        self.REMOTE_EVENTS_ENABLED = self.run_function("system.get_remote_events")
        self.SUBNET_NAME = self.run_function("system.get_subnet_name")
        self.KEYBOARD_DISABLED = self.run_function(
            "system.get_disable_keyboard_on_lock"
        )

    def tearDown(self):
        """
        Reset to original settings
        """
        if not self.ATRUN_ENABLED:
            atrun = "/System/Library/LaunchDaemons/com.apple.atrun.plist"
            self.run_function("service.stop", [atrun])

        self.run_function("system.set_remote_login", [self.REMOTE_LOGIN_ENABLED])
        self.run_function("system.set_remote_events", [self.REMOTE_EVENTS_ENABLED])
        self.run_function("system.set_subnet_name", [self.SUBNET_NAME])
        self.run_function(
            "system.set_disable_keyboard_on_lock", [self.KEYBOARD_DISABLED]
        )

    @pytest.mark.destructive_test
    @pytest.mark.slow_test
    def test_get_set_remote_login(self):
        """
        Test system.get_remote_login
        Test system.set_remote_login
        """
        # Normal Functionality
        self.assertTrue(self.run_function("system.set_remote_login", [True]))
        self.assertTrue(self.run_function("system.get_remote_login"))
        self.assertTrue(self.run_function("system.set_remote_login", [False]))
        self.assertFalse(self.run_function("system.get_remote_login"))

        # Test valid input
        self.assertTrue(self.run_function("system.set_remote_login", [True]))
        self.assertTrue(self.run_function("system.set_remote_login", [False]))
        self.assertTrue(self.run_function("system.set_remote_login", ["yes"]))
        self.assertTrue(self.run_function("system.set_remote_login", ["no"]))
        self.assertTrue(self.run_function("system.set_remote_login", ["On"]))
        self.assertTrue(self.run_function("system.set_remote_login", ["Off"]))
        self.assertTrue(self.run_function("system.set_remote_login", [1]))
        self.assertTrue(self.run_function("system.set_remote_login", [0]))

        # Test invalid input
        self.assertIn(
            "Invalid String Value for Enabled",
            self.run_function("system.set_remote_login", ["spongebob"]),
        )

    @pytest.mark.destructive_test
    @pytest.mark.slow_test
    def test_get_set_remote_events(self):
        """
        Test system.get_remote_events
        Test system.set_remote_events
        """
        # Normal Functionality
        self.assertTrue(self.run_function("system.set_remote_events", [True]))
        self.assertTrue(self.run_function("system.get_remote_events"))
        self.assertTrue(self.run_function("system.set_remote_events", [False]))
        self.assertFalse(self.run_function("system.get_remote_events"))

        # Test valid input
        self.assertTrue(self.run_function("system.set_remote_events", [True]))
        self.assertTrue(self.run_function("system.set_remote_events", [False]))
        self.assertTrue(self.run_function("system.set_remote_events", ["yes"]))
        self.assertTrue(self.run_function("system.set_remote_events", ["no"]))
        self.assertTrue(self.run_function("system.set_remote_events", ["On"]))
        self.assertTrue(self.run_function("system.set_remote_events", ["Off"]))
        self.assertTrue(self.run_function("system.set_remote_events", [1]))
        self.assertTrue(self.run_function("system.set_remote_events", [0]))

        # Test invalid input
        self.assertIn(
            "Invalid String Value for Enabled",
            self.run_function("system.set_remote_events", ["spongebob"]),
        )

    @pytest.mark.destructive_test
    @pytest.mark.slow_test
    def test_get_set_subnet_name(self):
        """
        Test system.get_subnet_name
        Test system.set_subnet_name
        """
        self.assertTrue(self.run_function("system.set_subnet_name", [SET_SUBNET_NAME]))
        self.assertEqual(self.run_function("system.get_subnet_name"), SET_SUBNET_NAME)

    @pytest.mark.slow_test
    def test_get_list_startup_disk(self):
        """
        Test system.get_startup_disk
        Test system.list_startup_disks
        Don't know how to test system.set_startup_disk as there's usually only
        one startup disk available on a system
        """
        # Test list and get
        ret = self.run_function("system.list_startup_disks")
        self.assertIsInstance(ret, list)
        self.assertIn(self.run_function("system.get_startup_disk"), ret)

        # Test passing set a bad disk
        self.assertIn(
            "Invalid value passed for path.",
            self.run_function("system.set_startup_disk", ["spongebob"]),
        )

    @skipIf(True, "Skip this test until mac fixes it.")
    def test_get_set_restart_delay(self):
        """
        Test system.get_restart_delay
        Test system.set_restart_delay
        system.set_restart_delay does not work due to an apple bug, see docs
        may need to disable this test as we can't control the delay value
        """
        # Normal Functionality
        self.assertTrue(self.run_function("system.set_restart_delay", [90]))
        self.assertEqual(self.run_function("system.get_restart_delay"), "90 seconds")

        # Pass set bad value for seconds
        self.assertIn(
            "Invalid value passed for seconds.",
            self.run_function("system.set_restart_delay", [70]),
        )

    @pytest.mark.slow_test
    def test_get_set_disable_keyboard_on_lock(self):
        """
        Test system.get_disable_keyboard_on_lock
        Test system.set_disable_keyboard_on_lock
        """
        # Normal Functionality
        self.assertTrue(
            self.run_function("system.set_disable_keyboard_on_lock", [True])
        )
        self.assertTrue(self.run_function("system.get_disable_keyboard_on_lock"))

        self.assertTrue(
            self.run_function("system.set_disable_keyboard_on_lock", [False])
        )
        self.assertFalse(self.run_function("system.get_disable_keyboard_on_lock"))

        # Test valid input
        self.assertTrue(
            self.run_function("system.set_disable_keyboard_on_lock", [True])
        )
        self.assertTrue(
            self.run_function("system.set_disable_keyboard_on_lock", [False])
        )
        self.assertTrue(
            self.run_function("system.set_disable_keyboard_on_lock", ["yes"])
        )
        self.assertTrue(
            self.run_function("system.set_disable_keyboard_on_lock", ["no"])
        )
        self.assertTrue(
            self.run_function("system.set_disable_keyboard_on_lock", ["On"])
        )
        self.assertTrue(
            self.run_function("system.set_disable_keyboard_on_lock", ["Off"])
        )
        self.assertTrue(self.run_function("system.set_disable_keyboard_on_lock", [1]))
        self.assertTrue(self.run_function("system.set_disable_keyboard_on_lock", [0]))

        # Test invalid input
        self.assertIn(
            "Invalid String Value for Enabled",
            self.run_function("system.set_disable_keyboard_on_lock", ["spongebob"]),
        )

    @skipIf(True, "Skip this test until mac fixes it.")
    def test_get_set_boot_arch(self):
        """
        Test system.get_boot_arch
        Test system.set_boot_arch
        system.set_boot_arch does not work due to an apple bug, see docs
        may need to disable this test as we can't set the boot architecture
        """
        # Normal Functionality
        self.assertTrue(self.run_function("system.set_boot_arch", ["i386"]))
        self.assertEqual(self.run_function("system.get_boot_arch"), "i386")
        self.assertTrue(self.run_function("system.set_boot_arch", ["default"]))
        self.assertEqual(self.run_function("system.get_boot_arch"), "default")

        # Test invalid input
        self.assertIn(
            "Invalid value passed for arch",
            self.run_function("system.set_boot_arch", ["spongebob"]),
        )


@pytest.mark.skip_unless_on_darwin
@pytest.mark.skip_if_not_root
class MacSystemComputerNameTest(ModuleCase):
    def setUp(self):
        self.COMPUTER_NAME = self.run_function("system.get_computer_name")
        self.wait_for_all_jobs()

    def tearDown(self):
        self.run_function("system.set_computer_name", [self.COMPUTER_NAME])
        self.wait_for_all_jobs()

    # A similar test used to be skipped on py3 due to 'hanging', if we see
    # something similar again we may want to skip this gain until we
    # investigate
    # @skipIf(salt.utils.platform.is_darwin() and six.PY3, 'This test hangs on OS X on Py3.  Skipping until #53566 is merged.')
    @pytest.mark.destructive_test
    @pytest.mark.slow_test
    def test_get_set_computer_name(self):
        """
        Test system.get_computer_name
        Test system.set_computer_name
        """
        log.debug("Set name is %s", SET_COMPUTER_NAME)
        self.assertTrue(
            self.run_function("system.set_computer_name", [SET_COMPUTER_NAME])
        )
        self.assertEqual(
            self.run_function("system.get_computer_name"), SET_COMPUTER_NAME
        )



import salt.modules.macdefaults as macdefaults
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MacDefaultsTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {macdefaults: {}}

    def test_write_default(self):
        """
        Test writing a default setting
        """
        mock = MagicMock()
        with patch.dict(macdefaults.__salt__, {"cmd.run_all": mock}):
            macdefaults.write("com.apple.CrashReporter", "DialogType", "Server")
            mock.assert_called_once_with(
                'defaults write "com.apple.CrashReporter" "DialogType" -string'
                ' "Server"',
                runas=None,
            )

    def test_write_with_user(self):
        """
        Test writing a default setting with a specific user
        """
        mock = MagicMock()
        with patch.dict(macdefaults.__salt__, {"cmd.run_all": mock}):
            macdefaults.write(
                "com.apple.CrashReporter", "DialogType", "Server", user="frank"
            )
            mock.assert_called_once_with(
                'defaults write "com.apple.CrashReporter" "DialogType" -string'
                ' "Server"',
                runas="frank",
            )

    def test_write_default_boolean(self):
        """
        Test writing a default setting
        """
        mock = MagicMock()
        with patch.dict(macdefaults.__salt__, {"cmd.run_all": mock}):
            macdefaults.write("com.apple.CrashReporter", "Crash", True, type="boolean")
            mock.assert_called_once_with(
                'defaults write "com.apple.CrashReporter" "Crash" -boolean "TRUE"',
                runas=None,
            )

    def test_read_default(self):
        """
        Test reading a default setting
        """
        mock = MagicMock()
        with patch.dict(macdefaults.__salt__, {"cmd.run": mock}):
            macdefaults.read("com.apple.CrashReporter", "Crash")
            mock.assert_called_once_with(
                'defaults read "com.apple.CrashReporter" "Crash"', runas=None
            )

    def test_read_default_with_user(self):
        """
        Test reading a default setting as a specific user
        """
        mock = MagicMock()
        with patch.dict(macdefaults.__salt__, {"cmd.run": mock}):
            macdefaults.read("com.apple.CrashReporter", "Crash", user="frank")
            mock.assert_called_once_with(
                'defaults read "com.apple.CrashReporter" "Crash"', runas="frank"
            )

    def test_delete_default(self):
        """
        Test delete a default setting
        """
        mock = MagicMock()
        with patch.dict(macdefaults.__salt__, {"cmd.run_all": mock}):
            macdefaults.delete("com.apple.CrashReporter", "Crash")
            mock.assert_called_once_with(
                'defaults delete "com.apple.CrashReporter" "Crash"',
                output_loglevel="debug",
                runas=None,
            )

    def test_delete_default_with_user(self):
        """
        Test delete a default setting as a specific user
        """
        mock = MagicMock()
        with patch.dict(macdefaults.__salt__, {"cmd.run_all": mock}):
            macdefaults.delete("com.apple.CrashReporter", "Crash", user="frank")
            mock.assert_called_once_with(
                'defaults delete "com.apple.CrashReporter" "Crash"',
                output_loglevel="debug",
                runas="frank",
            )



"""
    :codeauthor: Thomas Stoner <tmstoner@cisco.com>
"""

# Copyright (c) 2018 Cisco and/or its affiliates.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from tests.unit.modules.nxos.nxos_platform import NXOSPlatform


class N5KPlatform(NXOSPlatform):

    """Cisco Systems N5K Platform Unit Test Object"""

    chassis = "cisco Nexus 5672UP 16G-FC Chassis"

    # Captured output from: show install all impact kickstart <kimage> system <image>

    show_install_all_impact = """
Verifying image bootflash:/$KIMAGE for boot variable "kickstart".
[####################] 100% -- SUCCESS

Verifying image bootflash:/$IMAGE for boot variable "system".
[####################] 100% -- SUCCESS

Verifying image type.
[####################] 100% -- SUCCESS

Extracting "system" version from image bootflash:/$IMAGE.
[####################] 100% -- SUCCESS

Extracting "kickstart" version from image bootflash:/$KIMAGE.
[####################] 100% -- SUCCESS

Extracting "bios" version from image bootflash:/$IMAGE.
[####################] 100% -- SUCCESS

Performing module support checks.
[####################] 100% -- SUCCESS



Compatibility check is done:
Module  bootable          Impact  Install-type  Reason
------  --------  --------------  ------------  ------
     0       yes      disruptive         reset  ISSD is not supported and switch will reset with ascii configuration
     1       yes      disruptive         reset  ISSD is not supported and switch will reset with ascii configuration
     2       yes      disruptive         reset  ISSD is not supported and switch will reset with ascii configuration



Images will be upgraded according to following table:
Module             Image         Running-Version             New-Version  Upg-Required
------  ----------------  ----------------------  ----------------------  ------------
     0            system            $CVER             $NVER           $REQ
     0         kickstart            $CVER             $NVER           $REQ 
     0              bios      v0.1.9(03/09/2016)      v0.1.6(12/03/2015)            no
     0         power-seq    SF-uC:37, SF-FPGA:35    SF-uC:37, SF-FPGA:35            no
     0            iofpga               v0.0.0.39               v0.0.0.39            no
     1            iofpga               v0.0.0.18               v0.0.0.18            no
     2            iofpga               v0.0.0.18               v0.0.0.18            no

Warning : ISSD is not supported and switch will reset with ASCII configuration.
All incompatible configuration will be lost in the target release.
Please also refer the downgrade procedure documentation of the release for more details.
    """

    # Captured output from: install all kickstart <kimage> system <image> '''

    install_all_disruptive_success = """
Verifying image bootflash:/$KIMAGE for boot variable "kickstart".
[####################] 100% -- SUCCESS

Verifying image bootflash:/$IMAGE for boot variable "system".
[####################] 100% -- SUCCESS

Verifying image type.
[####################] 100% -- SUCCESS

Extracting "system" version from image bootflash:/$IMAGE.
[####################] 100% -- SUCCESS

Extracting "kickstart" version from image bootflash:/$KIMAGE.
[####################] 100% -- SUCCESS

Extracting "bios" version from image bootflash:/$IMAGE.
[####################] 100% -- SUCCESS

Performing module support checks.
[####################] 100% -- SUCCESS



Compatibility check is done:
Module  bootable          Impact  Install-type  Reason
------  --------  --------------  ------------  ------
     0       yes      disruptive         reset  ISSD is not supported and switch will reset with ascii configuration
     1       yes      disruptive         reset  ISSD is not supported and switch will reset with ascii configuration
     2       yes      disruptive         reset  ISSD is not supported and switch will reset with ascii configuration



Images will be upgraded according to following table:
Module             Image         Running-Version             New-Version  Upg-Required
------  ----------------  ----------------------  ----------------------  ------------
     0            system            $CVER             $NVER           $REQ
     0         kickstart            $CKVER            $NKVER          $KREQ
     0              bios      v0.1.9(03/09/2016)      v0.1.6(12/03/2015)            no
     0         power-seq    SF-uC:37, SF-FPGA:35    SF-uC:37, SF-FPGA:35            no
     0            iofpga               v0.0.0.39               v0.0.0.39            no
     1            iofpga               v0.0.0.18               v0.0.0.18            no
     2            iofpga               v0.0.0.18               v0.0.0.18            no

Warning : ISSD is not supported and switch will reset with ASCII configuration.
All incompatible configuration will be lost in the target release.
Please also refer the downgrade procedure documentation of the release for more details.

Install is in progress, please wait.

Performing runtime checks.
[####################] 100% -- SUCCESS

Setting boot variables.
[####################] 100% -- SUCCESS

Performing configuration copy.
[####################] 100% -- SUCCESS

Converting startup config.
[####################] 100% -- SUCCESS

Finishing the upgrade, switch will reboot in 10 seconds.
    """
